(function () {
    function googleChartsFormatter() {
        /*constructor*/
    }

    googleChartsFormatter.prepareGooglemap = function (mapObj) {
        var pointSize;
        var fillOpacity;
        var strokeColor;
        var pointsValArray = [];
        var objPoint;
        var objPointBigBorder;
        var objPointSmallBorder;
        var geoJson = {
            "type": "FeatureCollection",
            "features": []
        };

        var container = document.createElement('div');
        container.id = 'block';
        container.className = 'block';
        container.style.height = "310px";
        container.style.width = "800px";
        container.style.margin = "0";
        container.style.padding = "0";
        document.getElementsByTagName('body')[0].style.margin = "0";
        document.getElementsByTagName('body')[0].style.padding = "0";
        document.getElementsByTagName('body')[0].appendChild(container);

        var map;
        var mapId = 1;
        var mapContainer = document.getElementById('block');
        var params = {
            dark: false
        };

        mapObj.mapOptions.zoom = 1;
        mapTheme(params, mapObj);

        map = new google.maps.Map(mapContainer, mapObj.mapOptions);

        _.forEach(mapObj.points, function (point) {
            prepareMarkersObj(point, mapObj, map);
        });

        setVizualization(mapObj, map);
        map.data.addGeoJson(geoJson);

        function prepareMarkersObj(point, mapObj, map) {
            var pointValue = parseInt(point.label);
            var position = _.findIndex(mapObj.points, function(one) { return one.title === point.title; });
            point.map = map;

            if (mapObj.vizualization === "scatter") {
                fillOpacity = 1;
                strokeColor = "#000000";
                pointSize = 9;
            } else if (mapObj.vizualization === "sized_circles"){
                fillOpacity = .6;
                strokeColor = "same";
                pointSize = pointValue;
            } else if (mapObj.vizualization === "labeled_circles") {
                fillOpacity = 1;
                strokeColor = "#ffffff";
                pointSize = 400;
            }

            if (mapObj.vizualization === "labeled_circles") {
                objPointBigBorder = {
                    "type": "Feature",
                    "geometry": {
                        "type": "Point",
                        "coordinates": [point.position.lng, point.position.lat]
                    },
                    "properties": {
                        "label": point.label,
                        "title": point.title,
                        "value": pointValue,
                        "size": pointSize*1.85,
                        "border": strokeColor,
                        "opacity": 0,
                        "borderWeight": 1.5,
                        "position": position
                    }
                };

                objPointSmallBorder = {
                    "type": "Feature",
                    "geometry": {
                        "type": "Point",
                        "coordinates": [point.position.lng, point.position.lat]
                    },
                    "properties": {
                        "label": point.label,
                        "title": point.title,
                        "value": pointValue,
                        "size": pointSize*1.5,
                        "border": strokeColor,
                        "opacity": 0.6,
                        "borderWeight": 0,
                        "position": position
                    }
                };

                objPoint = {
                    "type": "Feature",
                    "geometry": {
                        "type": "Point",
                        "coordinates": [point.position.lng, point.position.lat]
                    },
                    "properties": {
                        "label": point.label,
                        "title": point.title,
                        "value": pointValue,
                        "size": pointSize,
                        "border": strokeColor,
                        "opacity": fillOpacity,
                        "borderWeight": 1.5,
                        "position": position
                    }
                };

                geoJson.features.push(objPointBigBorder, objPointSmallBorder, objPoint);
            } else {
                objPoint = {
                    "type": "Feature",
                    "geometry": {
                        "type": "Point",
                        "coordinates": [point.position.lng, point.position.lat]
                    },
                    "properties": {
                        "label": point.label,
                        "title": point.title,
                        "value": pointValue,
                        "size": pointSize,
                        "border": strokeColor,
                        "opacity": fillOpacity,
                        "borderWeight": .5
                    }
                };

                geoJson.features.push(objPoint);
            }
        }

        function setVizualization(mapObj, map) {
            if (mapObj.vizualization === "labeled_circles") {
                map.data.setStyle(function (feature) {
                    return ({
                        icon: setCircle(
                            feature.getProperty('value'),
                            feature.getProperty('size'),
                            feature.getProperty('border'),
                            feature.getProperty('opacity'),
                            feature.getProperty('borderWeight')
                        ),
                        title: feature.getProperty('title'),
                        label: {
                            color: "#ffffff",
                            fontFamily: "Verdana",
                            fontSize: "11px",
                            text: feature.getProperty('label')
                        },
                        zIndex: feature.getProperty('value') + feature.getProperty('position')
                    });
                });
            } else {
                map.data.setStyle(function (feature) {
                    return ({
                        icon: setCircle(
                            feature.getProperty('value'),
                            feature.getProperty('size'),
                            feature.getProperty('border'),
                            feature.getProperty('opacity'),
                            feature.getProperty('borderWeight')
                        ),
                        title: feature.getProperty('title'),
                        zIndex: feature.getProperty('value')
                    });
                });
            }
        }

        function setCircle(pointValue, pointSize, pointBorder, pointOpacity, borderWeight) {
            var utils = HighchartsFormatter.commonUtils();
            var colorPosition;
            var gradientPosition;
            var maxGradientPosition;
            var color;
            var minVal = getMinVal();
            var maxVal = getMaxVal();
            var minSettedVal = 0;
            var defaultPosition;
            var dataValue = pointValue / maxVal;

            if (mapObj.color_theme === "heatmap") {
                if (mapObj.colorAxis.stops.colors.length === 2) {
                    if (mapObj.colorAxis.stops.pMode === true) {
                        gradientPosition = mapObj.colorAxis.stops.colors[0].to;
                    } else {
                        if (typeof mapObj.colorAxis.stops.colors[0].from === "number") {
                            minSettedVal = mapObj.colorAxis.stops.colors[0].from / maxVal;
                        } else {
                            minSettedVal = minVal / maxVal;
                        }

                        if (mapObj.colorAxis.stops.colors[0].to === "Auto") {
                            gradientPosition = 0.5;
                        } else {
                            defaultPosition = mapObj.colorAxis.stops.colors[0].to;
                            if (_.isNumber(mapObj.colorAxis.stops.colors[1].to)) {
                                maxGradientPosition = mapObj.colorAxis.stops.colors[1].to;
                            } else {
                                maxGradientPosition = maxVal;
                            }
                            gradientPosition = defaultPosition / maxGradientPosition;
                            if (gradientPosition > 1) {
                                gradientPosition = 1;
                            }
                        }
                    }

                    var from = mapObj.colorAxis.stops.colors[0].color;
                    var to = mapObj.colorAxis.stops.colors[1].color;
                    var gradientStop = utils.getColorBetween(from, to, gradientPosition);

                    if (dataValue <= minSettedVal) {
                        color = from;
                    } else if (dataValue >= gradientPosition) {
                        color = utils.getColorBetween(gradientStop, to, 1 - dataValue);
                    } else if (dataValue < gradientPosition) {
                        colorPosition = _.round(dataValue / gradientPosition, 2);
                        color = utils.getColorBetween(from, gradientStop, 1 - colorPosition);

                    }
                } else {
                    var gradientPosition1;
                    var gradientPosition2;
                    var from1 = mapObj.colorAxis.stops.colors[0].color;
                    var to1 = mapObj.colorAxis.stops.colors[1].color;
                    var from2 = mapObj.colorAxis.stops.colors[1].color;
                    var to2 = mapObj.colorAxis.stops.colors[2].color;
                    var gradientStop1;
                    var gradientStop2;
                    var middelValue;
                    var middelPosition;

                    if (mapObj.colorAxis.stops.pMode === true) {
                        gradientPosition1 = mapObj.colorAxis.stops.colors[0].to;
                        gradientPosition2 = mapObj.colorAxis.stops.colors[1].to;
                    } else {
                        if (typeof mapObj.colorAxis.stops.colors[0].from === "number") {
                            minSettedVal = mapObj.colorAxis.stops.colors[0].from / maxVal;
                        } else {
                            minSettedVal = minVal / maxVal;
                        }

                        if (mapObj.colorAxis.stops.colors[0].to === "Auto" ) {
                            gradientPosition1 = 0.33;
                        }

                        if (mapObj.colorAxis.stops.colors[1].to === "Auto") {
                            gradientPosition2 = 0.66;
                        }

                        if (_.isNumber(mapObj.colorAxis.stops.colors[2].to)) {
                            maxGradientPosition = mapObj.colorAxis.stops.colors[2].to;
                        } else {
                            maxGradientPosition = maxVal;
                        }

                        if (typeof gradientPosition1 !== "number") {
                            gradientPosition1 = mapObj.colorAxis.stops.colors[0].to / maxGradientPosition;
                            if (gradientPosition1 > 1) {
                                gradientPosition1 = 1;
                            }
                        }

                        if (typeof gradientPosition2 !== "number") {
                            gradientPosition2 = mapObj.colorAxis.stops.colors[1].to / maxGradientPosition;
                            if (gradientPosition2 > 1) {
                                gradientPosition2 = 1;
                            }
                        }
                    }

                    middelValue = (gradientPosition1 + gradientPosition2) / 2;
                    middelPosition = middelValue / maxVal;
                    gradientStop1 = utils.getColorBetween(from1, to1, gradientPosition1);
                    gradientStop2 = utils.getColorBetween(from2, to2, gradientPosition2);

                    if (dataValue <= minSettedVal) {
                        color = from1;
                    } else if (dataValue < gradientPosition1) {
                        colorPosition = _.round(dataValue / gradientPosition1, 1);
                        color = utils.getColorBetween(from1, gradientStop1, 1 - colorPosition);
                    } else if (dataValue >= gradientPosition1 && dataValue < middelPosition) {
                        color = utils.getColorBetween(gradientStop1, to1, 1 - dataValue);
                    } else if (dataValue >= middelPosition && dataValue < gradientPosition2) {
                        color = utils.getColorBetween(to1, gradientStop2, 1 - dataValue);
                    } else if (dataValue >= gradientPosition2) {
                        color = utils.getColorBetween(gradientStop2, to2, 1 - dataValue);
                    }
                }
            } else {
                color = "#ff0000";
            }

            if (pointOpacity === 0) {
                pointBorder = color;
            }

            if (pointBorder === "same") {
                pointBorder = color;
                borderWeight = 2;
            }

            if (mapObj.vizualization === "sized_circles") {
                var scaleVal = 45/(maxVal-minVal)*(pointValue-maxVal)+50;
            } else {
                scaleVal = Math.sqrt(pointSize);
            }

            return {
                path: google.maps.SymbolPath.CIRCLE,
                fillColor: color,
                fillOpacity: pointOpacity,
                scale: scaleVal,
                strokeColor: pointBorder,
                strokeWeight: borderWeight
            };
        }

        function preparePoints() {
            _.forEach(mapObj.points, function(one){
                pointsValArray.push(parseInt(one.label));
            });
            return pointsValArray;
        }

        function getMaxVal() {
            var pointsVal = preparePoints();
            return _.max(pointsVal, function (value) {
                return value;
            })
        }

        function getMinVal() {
            var pointsVal = preparePoints();
            return _.min(pointsVal, function (value) {
                return value;
            })
        }

        function mapTheme(params, mapObj) {
            if (params.dark === true) {
                mapObj.mapOptions.styles = [
                    {
                        "featureType": "all",
                        "elementType": "labels.text.fill",
                        "stylers": [
                            {
                                "saturation": 36
                            },
                            {
                                "color": "#000000"
                            },
                            {
                                "lightness": 40
                            }
                        ]
                    },
                    {
                        "featureType": "all",
                        "elementType": "labels.text.stroke",
                        "stylers": [
                            {
                                "visibility": "on"
                            },
                            {
                                "color": "#000000"
                            },
                            {
                                "lightness": 16
                            }
                        ]
                    },
                    {
                        "featureType": "all",
                        "elementType": "labels.icon",
                        "stylers": [
                            {
                                "visibility": "off"
                            }
                        ]
                    },
                    {
                        "featureType": "administrative",
                        "elementType": "geometry.fill",
                        "stylers": [
                            {
                                "color": "#000000"
                            },
                            {
                                "lightness": 20
                            }
                        ]
                    },
                    {
                        "featureType": "administrative",
                        "elementType": "geometry.stroke",
                        "stylers": [
                            {
                                "color": "#000000"
                            },
                            {
                                "lightness": 17
                            },
                            {
                                "weight": 1.2
                            }
                        ]
                    },
                    {
                        "featureType": "administrative.country",
                        "elementType": "labels.text",
                        "stylers": [
                            {
                                "lightness": "11"
                            },
                            {
                                "saturation": "0"
                            },
                            {
                                "visibility": "simplified"
                            }
                        ]
                    },
                    {
                        "featureType": "landscape",
                        "elementType": "geometry",
                        "stylers": [
                            {
                                "color": "#000000"
                            },
                            {
                                "lightness": 20
                            },
                            {
                                "visibility": "on"
                            }
                        ]
                    },
                    {
                        "featureType": "poi",
                        "elementType": "geometry",
                        "stylers": [
                            {
                                "color": "#000000"
                            },
                            {
                                "lightness": 21
                            }
                        ]
                    },
                    {
                        "featureType": "road.highway",
                        "elementType": "geometry.fill",
                        "stylers": [
                            {
                                "color": "#121111"
                            },
                            {
                                "lightness": 17
                            }
                        ]
                    },
                    {
                        "featureType": "road.highway",
                        "elementType": "geometry.stroke",
                        "stylers": [
                            {
                                "color": "#000000"
                            },
                            {
                                "lightness": 29
                            },
                            {
                                "weight": 0.2
                            }
                        ]
                    },
                    {
                        "featureType": "road.highway",
                        "elementType": "labels.text.fill",
                        "stylers": [
                            {
                                "visibility": "off"
                            }
                        ]
                    },
                    {
                        "featureType": "road.arterial",
                        "elementType": "geometry",
                        "stylers": [
                            {
                                "color": "#000000"
                            },
                            {
                                "lightness": 18
                            }
                        ]
                    },
                    {
                        "featureType": "road.local",
                        "elementType": "geometry",
                        "stylers": [
                            {
                                "color": "#000000"
                            },
                            {
                                "lightness": 16
                            }
                        ]
                    },
                    {
                        "featureType": "transit",
                        "elementType": "geometry",
                        "stylers": [
                            {
                                "color": "#000000"
                            },
                            {
                                "lightness": 19
                            }
                        ]
                    },
                    {
                        "featureType": "water",
                        "elementType": "geometry",
                        "stylers": [
                            {
                                "color": "#0f252e"
                            },
                            {
                                "lightness": 17
                            }
                        ]
                    },
                    {
                        "featureType": "water",
                        "elementType": "geometry.fill",
                        "stylers": [
                            {
                                "saturation": "-52"
                            },
                            {
                                "lightness": "2"
                            }
                        ]
                    }
                ];
            } else {
                if (typeof mapObj.mapOptions.styles !== "undefined") {
                    delete mapObj.mapOptions.styles;
                }
            }
        }
    };

    window.googleChartsFormatter = googleChartsFormatter;

})();