(function () {
    function HighchartsFormatter() {
        /*constructor*/
    }

    HighchartsFormatter.prepareHeatmap = function (chartHolder) {
        var plotOptions = _.pick(chartHolder.plotOptions, "heatmap"),
            formatterUtils = HighchartsFormatter.commonUtils();

        chartHolder.plotOptions = plotOptions;

        var heatMap = chartHolder;

        if (heatMap.series && heatMap.series.length > 0) {
            var data = heatMap.series[0].data,
                dataInterval = (heatMap.xAxis.tickPositions) ? formatterUtils.getItemsInterval(data) : 1;

            heatMap.series = [{
                borderWidth: 1,
                data: data,
                dataLabels: {
                    color: '#000000',
                    enabled: true,
                    style: {
                        "textOutline": "none"
                    }
                }

            }];
            if (heatMap.xAxis.type && heatMap.xAxis.type === 'datetime') {
                _.extend(heatMap.series[0], {
                    colsize: formatterUtils.getColWidth(heatMap.xAxis.tickPositions, dataInterval)
                });
            } else {
                _.each(heatMap.series[0].data, function (item) {
                    item.x = item.x * 1;
                    item.y = item.y * 1;
                })
            }
            if (heatMap.colorAxis.stops.pMode) {
                heatMap.colorAxis.stops = formatterUtils.preparePercentsStops(heatMap.colorAxis.stops.colors);
            } else {
                var max = (getUserMaxValue() === 'Max') ? formatterUtils.getMaxCell(heatMap.series[0].data).value : getUserMaxValue(),
                    min = (getUserMinValue() === 'Min') ? formatterUtils.getMinCell(heatMap.series[0].data).value : getUserMinValue();
                _.extend(heatMap.colorAxis, {
                    max: max,
                    min: min,
                    stops: formatterUtils.prepareUserStops(heatMap.series[0].data, heatMap.colorAxis.stops.colors, min, max)
                });
            }
            return heatMap;
        }

        function getUserMaxValue() {
            return _.findLast(heatMap.colorAxis.stops.colors).to;
        }

        function getUserMinValue() {
            return _.first(heatMap.colorAxis.stops.colors).from;
        }

        return heatMap;
    };

    HighchartsFormatter.phantomHeatmapFormatter = function (chartObj) {
        var formattedObj = HighchartsFormatter.prepareHeatmap(chartObj);
        var cats = [];
        formattedObj.yAxis.categories.forEach(function(cat){
            if(cat.length > 12){
                cat = cat.substr(0, 12) + '...';
            }
            cats.push(cat);
        });
        formattedObj.chart.backgroundColor = '#FFFFFF';
        formattedObj.yAxis.categories = cats;
        return JSON.stringify(formattedObj);
    }

    HighchartsFormatter.toggleHeatmapCellValue = function () {
        if (this.cell.point.dataLabel) {
            var valuesGroupElem = this.cell.point.dataLabel.parentGroup.element,
                elem = this.cell.point.graphic.element,
                dataLabel = this.label,
                heatMapHideValues;

            (dataLabel.width > gelHeatmapCellWidth(elem))
                ? heatMapValuesVisability.push(false)
                : heatMapValuesVisability.push(true);

            heatMapHideValues = heatMapValuesVisability.some(checkFalse);

            (heatMapHideValues)
                ? valuesGroupElem.classList.add('hidden-values')
                : valuesGroupElem.classList.remove('hidden-values');
        }

        function gelHeatmapCellWidth(elem) {
            return parseInt(elem.getAttribute('width'));
        }

        function checkFalse(element) {
            return element === false;
        }

    };

    HighchartsFormatter.prepareStackedGroupedColumn = function (chartHolder, activatePhantomMode) {
        var stackedGroupedColumn = Highcharts.stackedGroupedColumn;

        var parsedSeriesData = stackedGroupedColumn.parseData(chartHolder),
            stackIndex = stackedGroupedColumn.utils.setStackIndex(parsedSeriesData.stackIndex),
            containerWidth = chartHolder.chart.width,
            chartSize = stackedGroupedColumn.getContainerWidth(stackIndex, chartHolder.xAxis.tickPositions, chartHolder);

        stackedGroupedColumn.utils.setPhantomMode(activatePhantomMode);

        var anotherHolder = {
            series: parsedSeriesData.series,
            chart: {
                type: 'column',
                width: (chartSize.chartWidth > containerWidth) ? chartSize.chartWidth : containerWidth,
                events: {
                    load: function () {
                        stackedGroupedColumn.addLabelsBackgrounds.call(this);
                        stackedGroupedColumn.addStackLabelsBackgrounds.call(this, chartHolder, stackIndex);
                    },
                    redraw: function () {
                        stackedGroupedColumn.addLabelsBackgrounds.call(this);
                        stackedGroupedColumn.addStackLabelsBackgrounds.call(this, chartHolder, stackIndex);
                    }
                }
            },
            legendTop: {
                width: containerWidth
            },
            legendBottom: {
                width: containerWidth
            },
            xAxis: {
                labels: {
                    rotation: 0
                }
            },
            tooltip: {
                formatter: stackedGroupedColumn.stackedTooltip
            },
            plotOptions: {
                series: {
                    pointWidth: chartSize.pointWidth,
                    groupPadding: chartSize.groupPadding,
                    events: {
                        legendItemClick: stackedGroupedColumn.legendItemClick
                    },
                    pointPlacement: chartSize.pointPlacement
                }
            },
            yAxis: {
                stackLabels: {
                    formatter: stackedGroupedColumn.stackedYLabels
                }
            }
        };

        if (activatePhantomMode) {
            anotherHolder.chart.backgroundColor = '#FFFFFF';
        }

        var combinedChartHolder = Highcharts.merge(chartHolder, anotherHolder);
        combinedChartHolder.legend = _.pick(combinedChartHolder.legend, ['enabled']);
        combinedChartHolder.tooltip = _.pick(combinedChartHolder.tooltip, 'formatter');
        combinedChartHolder.plotOptions = _.pick(combinedChartHolder.plotOptions, 'series');

        return combinedChartHolder;
    };

    HighchartsFormatter.prepareGauge = function (chartHolder) {
        var anotherHolder = {
            xAxis: {
                tickPositioner: function() {
                    return [this.min, this.max];
                }
            }
        };
        var combinedChartHolder = Highcharts.merge(chartHolder, anotherHolder);

        return combinedChartHolder;
    };

    HighchartsFormatter.phantomStackedGroupedColumnFormatter = function (chartObj) {
        var formattedObj = HighchartsFormatter.prepareStackedGroupedColumn(chartObj, true);
        return JSON.stringify(formattedObj);
    };

    HighchartsFormatter.commonUtils = function () {
        return {
            preparePercentsStops: preparePercentsStops,
            prepareUserStops: prepareUserStops,
            getColorBetween: getColorBetween,
            getHex: getHex,
            linearRange: linearRange,
            getMaxCell: getMaxCell,
            getMinCell: getMinCell,
            getColWidth: getColWidth,
            getItemsInterval: getItemsInterval
        };

        /* Utils */

        function preparePercentsStops(stops) {
            var stopsArr = [];
            if (stops.length > 2) {
                _.each(stops, function (stop, index) {
                    if (index == 1) {
                        stopsArr.push([(stop.from / 2), getCalculatedColor(stops, index)]);
                        stopsArr.push([stop.from, stops[index].color]);
                    } else if (index == 2) {
                        stopsArr.push([stop.from + ((1 - stop.from) / 2), getCalculatedColor(stops, index)]);
                    } else {
                        stopsArr.push([stop.from, getCalculatedColor(stops, index)]);
                    }
                });
            } else {
                _.each(stops, function (stop, index) {
                    stopsArr.push([stop.from, getCalculatedColor(stops, index)]);
                });
            }

            stopsArr.push([1, _.findLast(stops).color]);

            return stopsArr;
        }

        function prepareUserStops(data, stops, min, max) {
            var stopsArr = [],
                step = _.round(1 / stops.length, 2);

            if (stops.length > 2) {
                _.each(stops, function (stop, index) {
                    var currentStopPercent = index * step;
                    if (stop.from === 'Min') {
                        stopsArr.push([linearRange(getMinCell(data).value, min, max), stop.color]);
                    } else {
                        var stopPosition = (stop.from === 'Auto') ? currentStopPercent : linearRange(stop.from, min, max);
                        if (index == 1) {
                            stopsArr.push([(stopPosition) / 2, getCalculatedColor(stops, index)]);
                            stopsArr.push([(stopPosition), stops[index].color]);
                        } else if (index == 2) {
                            stopsArr.push([stopPosition + ((1 - stopPosition) / 2), getCalculatedColor(stops, index)]);
                        } else {
                            stopsArr.push([stopPosition, getCalculatedColor(stops, index)]);
                        }
                    }
                });
            } else {
                _.each(stops, function (stop, index) {
                    if (stop.from === 'Min') {
                        stopsArr.push([linearRange(getMinCell(data).value, min, max), stop.color]);
                    } else if (stop.from === 'Auto') {
                        stopsArr.push([step * index, getCalculatedColor(stops, index)]);
                    } else {
                        stopsArr.push([linearRange(stop.from, min, max), getCalculatedColor(stops, index)]);
                    }
                });
            }

            stopsArr.push([1, _.findLast(stops).color]);

            return stopsArr;
        }

        function getItemsInterval(items) {
            if (items.length) {
                return items[1].x - items[0].x;
            }
        }

        function getColWidth(positions, dataInterval) {
            if (positions && positions.length) {
                var ticksInterval = positions[1] - positions[0],
                    divider = ticksInterval / (dataInterval || 1);
                return ticksInterval / divider;
            }
            return 1;
        }

        function getMaxCell(data) {
            return _.max(data, function (i) {
                return i.value;
            })
        }

        function getMinCell(data) {
            return _.min(data, function (i) {
                return i.value;
            })
        }

        function getCalculatedColor(stops, index) {
            if (index !== 0) {
                return getColorBetween(stops[index - 1].color, stops[index].color, 0.5);
            }
            return stops[index].color;
        }

        function getColorBetween(c1, c2, point) {
            r = Math.ceil(parseInt(c1.substring(1, 3), 16) * point + parseInt(c2.substring(1, 3), 16) * (1 - point)),
                g = Math.ceil(parseInt(c1.substring(3, 5), 16) * point + parseInt(c2.substring(3, 5), 16) * (1 - point)),
                b = Math.ceil(parseInt(c1.substring(5, 7), 16) * point + parseInt(c2.substring(5, 7), 16) * (1 - point));
            return '#' + getHex(r) + getHex(g) + getHex(b);
        }

        function getHex(_x) {
            var x = _x.toString(16);
            return (x.length == 1) ? '0' + x : x;
        }

        function linearRange(x, min, max) {
            var divider = max - min;
            return _.round(((x - min) / (divider || 1)), 2);
        }
    };

    window.HighchartsFormatter = HighchartsFormatter;

})();
