function getName() {
	return "aws_elb";
}
xplgRegisterSystemDef({
	"xplg_system_def": {
		"uid": "aws_elb",
		"apptags": "",
		"displayName": "AWS ELB",
		"typetag": "aws,elb",
		"name": "aws_elb",
		"description": "",
		"type": "cloud",
		"doclink":  "http://docs.aws.amazon.com/elasticloadbalancing/latest/classic/access-log-collection.html#access-log-file-format",
		"version": "current",
		"xwikilink": "",
		"supportedDataSources": "localFolder,ssh,s3",
		"example": "",
		"logtypes": [{
				"log": {
					"parser_uid": "uid",
					"xdoclink": "",
					"defaultPattern": "timestamp elb client:port backend:port request_processing_time backend_processing_time response_processing_time elb_status_code backend_status_code received_bytes sent_bytes \"request\" \"user_agent\" ssl_cipher ssl_protocol",
					"howto": "The ELB access logs are placed in a S3 bucket that was configured, usually with the name structure: {string}.log",
					"typetag": "aws,elb,access",
					"name": "Access",
					"custom_parser_location": "<LINK to JS>",
					"defaultXpoLogPattern": "NONE",
					"parser_type": "default",
					"xwikilink": "http://wiki.xpolog.com/display/XPOL/Amazon+ELB",
					"type_uid": "elb_access",
					"parser_specification": {
						"defaultPattern": "timestamp elb client:port backend:port request_processing_time backend_processing_time response_processing_time elb_status_code backend_status_code received_bytes sent_bytes \"request\" \"user_agent\" ssl_cipher ssl_protocol",
						"parserparam": "",
						"defaultXpoLogPattern": "NONE",
						"externalJS": "link or code",
						"postBuild": function (result) {
							if (result != "") {
								result = result + "{eoe}"
							}
							return result;
						},
						"patternMatch": "\\S+",
						"filters":
						[{
								"filter": [{
										"match": "timestamp"
									}
								],
								"type": "Date",
								"function": {
									"filedName": "Date",
									"datePattern": "yyyy-MM-dd'T'HH:mm:ss.SSSSSS'Z'"
								}
							}, {
								"filter": [{
										"match": "type"
									}
								],
								"type": "CustomParse",
                                "function": {
                                    "custom_parse_function" : function (params){
                                        return "{choice:Type,ftype=type;,http;https;h2;ws;wss}";
                                    }
                                }
							}, {
								"filter": [{
										"match": "elb"
									}
								],
								"type": "Text",
								"function": {
									"filedName": "ELB-Name",
									"ftype": "elb"
								}
							}, {
								"filter": [{
										"match": "client:port"
									}
								],
								"type": "CustomParse",
                                "function": {
                                    "custom_parse_function" : function (params){
                                        return "{geoip:Client IP,ftype=remoteip}:{number:Client Port,ftype=remoteport}";
                                    }
                                }
							}, {
								"filter": [{
										"match": "backend:port"
									}
								],
								"type": "CustomParse",
                                "function": {
                                    "custom_parse_function" : function (params){
                                        return "{geoip:BackEnd IP,ftype=backendip}:{number:BackEnd Port,ftype=backendport}";
                                    }
                                }
							}, {
								"filter": [{
										"match": "request_processing_time"
									}
								],
								"type": "CustomParse",
                                "function": {
                                    "custom_parse_function" : function (params){
                                        return "{number:Request Processing Time (Sec),ftype=processrequestsecs}{expression:Request Processing Time (Ms),ftype=processrequestmilli,Request Processing Time (Sec) * 1000}";
                                    }
                                }
							}, {
								"filter": [{
										"match": "backend_processing_time"
									}
								],
								"type": "CustomParse",
                                "function": {
                                    "custom_parse_function" : function (params){
                                        return "{number:BackEnd Processing Time (Sec),ftype=backendprocessrequestsecs}{expression:BackEnd Processing Time (Ms),ftype=backendprocessrequestmilli,BackEnd Processing Time (Sec) * 1000}";
                                    }
                                }
							}, {
								"filter": [{
										"match": "target_processing_time"
									}
								],
								"type": "CustomParse",
                                "function": {
                                    "custom_parse_function" : function (params){
                                        return "{number:Target Processing Time (Sec),ftype=targetresponseprocessingtimesecs}{expression:Target Response Processing Time (Ms),ftype=targetresponseprocessingtimemilli,Target Response Processing Time (Sec) * 1000}";
                                    }
                                }
							}, {
								"filter": [{
										"match": "response_processing_time"
									}
								],
								"type": "CustomParse",
                                "function": {
                                    "custom_parse_function" : function (params){
                                        return "{number:Response Processing Time (Sec),ftype=responseprocessingtimesecs}{expression:Response Processing Time (Ms),ftype=responseprocessingtimemilli,Response Processing Time (Sec) * 1000}";
                                    }
                                }
							}, {
								"filter": [{
										"match": "elb_status_code"
									}
								],
								"type": "Text",
								"function": {
									"filedName": "ELB Status Code",
									"ftype": "elbstatuscode"
								}
							}, {
								"filter": [{
										"match": "backend_status_code"
									}
								],
								"type": "Text",
								"function": {
									"filedName": "BackEnd Status Code",
									"ftype": "respstatus"
								}
							}, {
								"filter": [{
										"match": "target_status_code"
									}
								],
								"type": "Text",
								"function": {
									"filedName": "Target Status Code",
									"ftype": "respstatus"
								}
							}, {
								"filter": [{
										"match": "received_bytes"
									}
								],
								"type": "Number",
								"function": {
									"filedName": "Received Bytes",
									"ftype": "bytesreceived"
								}
							}, {
								"filter": [{
										"match": "sent_bytes"
									}
								],
								"type": "Number",
								"function": {
									"filedName": "Bytes Sent",
								    "ftype": "bytesent"
								}
							}, {
								"filter": [{
										"match": "\"request\""
									}
								],
								"type": "CustomParse",
								"function": {
									"custom_parse_function": function (regexp_params) {
										return "\"{choice:Method,ftype=reqmethod;,GET;POST;HEAD} {url:URL,paramsFtype=querystring;ftype=requrl;paramsName=Query;,} {text:Request Protocol,ftype=reqprotocol;,}\"";
									}
								}
							}, {
								"filter": [{
										"match": "\"user_agent\""
									}
								],
								"type": "CustomParse",
								"function": {
									"custom_parse_function": function (regexp_params) {
										return "\"{text:User-agent,ftype=useragent}\"";
									}
								}
							}, {
								"filter": [{
										"match": "ssl_cipher"
									}
								],
								"type": "Text",
								"function": {
									"filedName": "SSL Cipher",
									"ftype": "sslcipher"
								}
							}, {
								"filter": [{
										"match": "ssl_protocol"
									}
								],
								"type": "Text",
								"function": {
									"filedName": "SSL Protocol",
									"ftype": "sslprotocol"
								}
							}, {
								"filter": [{
										"match": "target_group_arn"
									}
								],
								"type": "Text",
								"function": {
									"filedName": "Target Group arn",
									"ftype": "targetgrouparn"
								}
							}, {
								"filter": [{
										"match": "\"trace_id\""
									}
								],
								"type": "CustomParse",
								"function": {
									"custom_parse_function": function (regexp_params) {
										return "\"{text:Trace ID,ftype=traceid}\"";
									}
								}
							}, {
								"filter": [{
										"match": "\"domain_name\""
									}
								],
								"type": "CustomParse",
								"function": {
									"custom_parse_function": function (regexp_params) {
										return "\"{text:Domain,ftype=domain}\"";
									}
								}
							}, {
								"filter": [{
										"match": "\"chosen_cert_arn\""
									}
								],
								"type": "CustomParse",
								"function": {
									"custom_parse_function": function (regexp_params) {
										return "\"{text:Chosen Cert arn,ftype=chosencertarn}\"";
									}
								}
							}, {
								"filter": [{
										"match": "matched_rule_priority"
									}
								],
								"type": "Text",
								"function": {
									"filedName": "Matched Rule Priority",
									"ftype": "matchedrulepriority"
								}
							}, {
								"filter": [{
										"match": "request_creation_time"
									}
								],
								"type": "Date",
								"function": {
									"filedName": "Request Creation Time",
									"datePattern": "yyyy-MM-dd'T'HH:mm:ss z"
								}
							}, {
								"filter": [{
										"match": "\"actions_executed\""
									}
								],
								"type": "CustomParse",
								"function": {
									"custom_parse_function": function (regexp_params) {
										return "\"{text:Actions Executed,ftype=actionsexecuted}\"";
									}
								}
							}, {
								"filter": [{
										"match": "\"redirect_url\""
									}
								],
								"type": "CustomParse",
								"function": {
									"custom_parse_function": function (regexp_params) {
										return "\"{text:Redirect URL,ftype=redirecturl}\"";
									}
								}
							},{
								"filter": [{
										"match": "\"error_reason\""
									}
								],
								"type": "CustomParse",
								"function": {
									"custom_parse_function": function (regexp_params) {
										return "\"{text:Error Reason,ftype=errorreason}\"";
									}
								}
							}
						]
					}
				}
			}
		]
	}
});
