function runSearch(subQuery, restriction, strategy) {
    const query = document.querySelector('.auto-complete-editor__field');

    if (subQuery && (!restriction || !query.innerText.match(restriction))) {
        switch (strategy) {
            case 'search_trix_action_strategy':
                runSearchTrixAction(subQuery);
                break;
            case 'where_merge_strategy':
                runWhereComparison(subQuery);
                break;
            default:
                runSearchCommon(subQuery);
                break;
        }
    }
}

function runSearchCommon(subQuery) {
    const query = document.querySelector('.auto-complete-editor__field');
    const searchBtn = document.querySelector('.search-btn > .vm');

    query.append(subQuery);
    searchBtn.click();
}


function runSearchTrixAction(subQuery) {
    const searchBtnEl = document.querySelector('.search-btn > .vm');
    const queryEl = document.querySelector('.auto-complete-editor__field');
    const query = document.querySelector('.auto-complete-editor__field').innerText;
    const trixQueryPrefix = ' | trix ';
    const whiteSpaceRegExp = /\s/g;
    const trixEnrichActionRegExp = /cep\..+\.all/g;
    const isTrixQueryRunning = query.replace(whiteSpaceRegExp, '').includes('|trix');

    if (isTrixQueryRunning) {
        let trixQueryFunction = subQuery.substring(0, subQuery.indexOf(' ', 1)).replace(whiteSpaceRegExp, '');

        if (trixQueryFunction.match(trixEnrichActionRegExp)) {
            trixQueryFunction = trixEnrichActionRegExp;
        }

        // if current trix query already use subquery that we're going to add -> update it
        let trixQueryFunctionStartPos = query.search(trixQueryFunction);
        if (whiteSpaceRegExp.exec(query[trixQueryFunctionStartPos - 1])) {
            trixQueryFunctionStartPos -= 1;
        }
        if (trixQueryFunctionStartPos >= 0) {
            const endPos = query.indexOf(')', trixQueryFunctionStartPos);
            queryEl.innerHTML = query.substring(0, trixQueryFunctionStartPos) + subQuery + query.substring(endPos + 1);
        } else {
            // if current trix query doesn't use subquery that we're goind to add -> add it
            queryEl.append(subQuery);
        }
    } else {
        // if another from trix query -> add trix prefix and subquery
        queryEl.append(trixQueryPrefix + subQuery);
    }

    searchBtnEl.click();
}

function runWhereComparison(subQuery) {
    const query = document.querySelector('.auto-complete-editor__field');
    const searchBtn = document.querySelector('.search-btn > .vm');
    const queryText = query.innerText;
    const whereMatches = queryText.match(/\|\s?where/gi);
    let replaceWhereWithAnd = false;

    if (whereMatches.length) {
        const lastMatch = whereMatches[whereMatches.length - 1];
        const queryTextAfterLastWhere = queryText.substring(queryText.lastIndexOf(lastMatch) + lastMatch.length);
        if (!queryTextAfterLastWhere.includes('|')) {
            replaceWhereWithAnd = true;
        }
    }

    if (replaceWhereWithAnd) {
        subQuery = subQuery.replace('| where', 'and');
    }

    query.append(subQuery);
    searchBtn.click();
}
