import {
    AfterViewInit,
    ChangeDetectionStrategy, ChangeDetectorRef,
    Component,
    ElementRef, EventEmitter, HostListener,
    Input, OnChanges,
    OnDestroy,
    OnInit, Output, Renderer2, SimpleChanges, ViewChild
} from '@angular/core';
import {XplgContainerAbstract} from '../../xplg-container.abstract';
import {XplgContainerDiagramRestService} from '../../services/xplg-container-diagram-rest.service';
import {CommunicationService} from '../../services/communication.service';
import {DomSanitizer} from '@angular/platform-browser';
import {Subscription} from 'rxjs';

@Component({
    selector: 'xplg-container-diagram',
    templateUrl: './xplg-container-diagram.component.html',
    styleUrls: ['./xplg-container-diagram.component.scss'],
    providers: [CommunicationService],
    changeDetection: ChangeDetectionStrategy.OnPush
})
export class XplgContainerDiagramComponent extends XplgContainerAbstract implements OnInit, OnChanges, OnDestroy, AfterViewInit {
    private deregisterClickListener: (() => void) | undefined;
    private subsHolder: {[key: string]: Subscription} = {};

    @Input() data: any;
    @Input() diagramType: 'narrow' | 'advanced' = 'advanced';
    @Input() selectedNodeId: number = -1;

    @Output() restConfig = new EventEmitter();

    @ViewChild('diagramContainer') diagramContainer: ElementRef<HTMLIFrameElement> | undefined;

    constructor(private rest: XplgContainerDiagramRestService,
                private communication: CommunicationService,
                private sanitizer: DomSanitizer,
                private cdr: ChangeDetectorRef,
                private renderer: Renderer2) {
        super();
    }

    @HostListener('window:message', ['$event'])
    private message(event: MessageEvent) {
        this.communication.embeddedInstanceAction(event.data);
    }

    ngOnInit(): void {
        this.subsHolder['restConfSubs'] = this.communication.restConfig$.subscribe(conf => {
            this.restConfig.emit(conf);
        });
        this.embeddedConf = {src: this.sanitizer.bypassSecurityTrustResourceUrl('http://localhost:4200/')};
        this.communication.theme = 'default';
    }

    ngOnChanges(changes: SimpleChanges): void {
        if (changes['data']) {
            this.communication.data = changes['data']?.currentValue;
        }
    }

    ngAfterViewInit(): void {
        if (this.diagramContainer) {
            this.communication.target = this.diagramContainer;
        }
        this.communication.diagramType = this.diagramType;
        this.communication.selectedNodeId = this.selectedNodeId;
    }

    init(): void {
        if (this.diagramContainer?.nativeElement?.contentWindow) {
            this.deregisterClickListener = this.renderer
                .listen(this.diagramContainer?.nativeElement?.contentWindow.document, 'click', () => {
                    const customClick = new Event('click', {bubbles: true});
                    this.diagramContainer?.nativeElement.dispatchEvent(customClick);
                });
        }
    }

    reset(): void {
        this.communication.fireEvent({
            type: 'toggleNode',
            state: false
        });
    }

    protected initEmbeddedInstance(): void {
    }

    protected refreshEmbeddedInstance(): void {
    }

    protected getData(): void {
    }

    protected destroyEmbeddedInstance(): void {
    }

    protected listenEmbeddedInstance(): void {
    }

    ngOnDestroy(): void {
        if (this.deregisterClickListener) {
            this.deregisterClickListener();
        }
        for (const key of Object.keys(this.subsHolder)) {
            this.subsHolder[key].unsubscribe();
        }
    }
}
