export type CommunicationEvent = IDiagramLoaded
    | IRefreshDiagram
    | IDestroyDiagram
    | IInjectDiagramData
    | IDiagramRequest
    | IDiagramUIEvent
    | IDiagramAddNode
    | IDiagramTheme
    | IDiagramToggleNode;

/**
 * Notify XPLG Container embedded Diagram ready
 */
interface IDiagramLoaded extends ICommunicationEvent {
    type: 'loaded';
}

/**
 * Refresh embedded Diagram from XPLG Container
 */
interface IRefreshDiagram extends ICommunicationEvent {
    type: 'refresh';
}

/**
 * Destroy Refresh embedded Diagram from XPLG Container
 */
interface IDestroyDiagram extends ICommunicationEvent {
    type: 'destroy';
}

/**
 * Inject embedded Diagram data from XPLG Container
 */
interface IInjectDiagramData extends ICommunicationEvent {
    type: 'injectData';
    data: IXPLGData;
}

/**
 * Request Diagram data throw XPLG Container
 */
interface IDiagramRequest extends ICommunicationEvent {
    type: 'requestData';
    params: {
        url: string;
        method: 'post' | 'get' | 'update' | 'delete';
        queryParams?: {[key: string]: any};
        body: {[key: string]: any};
    }
}

/**
 * Handle DiagramEvent trow XPLG Container
 */
interface IDiagramUIEvent extends ICommunicationEvent {
    type: 'uiEvent';
    params: IXPLGEvent
}

/**
 * Handle DiagramEvent trow XPLG Container
 */
interface IDiagramAddNode extends ICommunicationEvent {
    type: 'addNode';
}

interface IDiagramTheme extends ICommunicationEvent {
    type: 'setTheme';
    theme: Theme;
}

interface IDiagramToggleNode extends ICommunicationEvent {
    type: 'toggleNode';
    state: boolean;
    targetId?: number;
}

interface ICommunicationEvent {
    theme?: Theme;
    callback?: () => void;
}

interface IXPLGData {
    dataType: 'node' | 'edge' | 'full';
    data: any;
}

export interface IXPLGEvent {
    type: 'click' | 'doubleClick' | 'rightClick' | 'mouseUp' | 'mouseDown' | 'drag';
    target: HTMLElement; // the target element on which the event happened
    view: HTMLElement; // the view on which the event happened
    time: string; // the time of the event,
    data: {[key: string]: any};
    coords: {
        x: number;
        y: number;
        offsetX: number;
        offsetY: number;
    }
    keyboard: {
        key: string;
        alt: boolean;
        shift: boolean;
        ctrl: boolean;
    },
    drag: any
}

export type Theme = 'default' | 'dark';
