export interface IXplgGraph {
    data: IData;
    ui?: IUi;
    behavior: IBehavior;
    grid: IGrid;
}

interface IData {
    manager?: {
        type: string;
        remoteRequestParams: any;
    };
    nodes: INode[];
    edges?: IEdge[];
}

export interface IEdge {
    id: EdgeId;
    style: CSSStyleDeclaration;
    styleId: 'edge_oneWay' | 'edge_twoWay',
    sourceId: SourceId;
    targetId: TargetId;
    data: [
        {
            type: EdgeType;
            value: string;
            uiPrefs: Object;
            position: IPosition;
        }
    ]
}

export interface INode {
    id: NodeId;
    style: CSSStyleDeclaration;
    styleId: 'node_groups' | 'node_logs' | 'node_hosts' | 'node_logIds';
    data: INodeData[];
    incomingEdgeIds: NodeId[];
    outgoingEdgeIds: EdgeId[];
    uiPrefs?: {
        tooltip?: string[];
        backgroundColor?: string;
        iconBackground?: string;
        color?: string;
    }
}

interface INodeData {
    type: NodeType;
    value: string;
    tooltip: string;
    uiPrefs: {
        color?: string;
        backgroundColor?: string;
        iconColor?: string;
    };
    position: IPosition;
}

export interface INodeWrapper {
    nodeType: 'node' | 'group';
    styleId: 'node_groups' | 'node_logs' | 'node_hosts' | 'node_logIds';
    node?: INode;
    nodes?: Array<INode>;
    tooltip?: string;
    connectorPosition?: 'both' | 'left' | 'right';
}

interface IUi {
    styles: Object
}

interface IBehavior {
    actions: IActions;
}

export interface IActions extends Object{
}

interface IGrid {
    type: GraphType;
}

interface IAlign {
    horizontal: Align;
    vertical: Align;
}

interface IPosition {
    margin: {
        left?: number;
        top?: number;
        right?: number;
        bottom?: number;
    },
    align: IAlign;
}

export type GraphType = 'static' | 'editable';

type NodeId = string;
type EdgeId = string;
type SourceId = string;
type TargetId = string;
type Align = 'center' | 'left' | 'right';
type NodeType = 'text' | 'html' | 'icon' | 'rest';
type EdgeType = 'text' | 'html' | 'icon' | 'rest';
