import {Injectable} from '@angular/core';
import {IEdge, INode} from '../models/xplg-graph.model';

type ConnectorRight = 'right';
type ConnectorLeft = 'left';
type ConnectorBoth = 'both';
type ConnectorPosition = ConnectorBoth | ConnectorLeft | ConnectorRight | null;

@Injectable()
export class DiagramService {
    static oneWayConnector: IEdge['styleId'] = 'edge_oneWay';
    static twoWaysConnector: IEdge['styleId'] = 'edge_twoWay';
    static connectorRight: ConnectorRight = 'right';
    static connectorLeft: ConnectorLeft = 'left';
    static connectorBoth: ConnectorBoth = 'both';

    static connectorPosition(node: INode, edges: IEdge[], index: number, arr: INode[]): ConnectorPosition {
        const prevNode = arr[index - 1];
        const edge = edges.find(edge => edge.id === node.outgoingEdgeIds[0]);
        const prevEdge = edges.find(edge => edge.id === prevNode?.outgoingEdgeIds[0]);
        if ((edge?.styleId === DiagramService.oneWayConnector || edge?.styleId === DiagramService.twoWaysConnector)
            && prevEdge?.styleId === DiagramService.twoWaysConnector) {
            return DiagramService.connectorBoth;
        } else if (!edge && prevEdge?.styleId === DiagramService.twoWaysConnector) {
            return DiagramService.connectorLeft;
        } else if (!edge && prevEdge?.styleId === DiagramService.oneWayConnector) {
            return null;
        } else {
            return DiagramService.connectorRight;
        }
    }
}
