import {Injectable} from '@angular/core';
import {Observable, of} from 'rxjs';
import {CommunicationEvent} from '../models/communication-event';
import {DiagramService} from './diagram.service';

@Injectable()
export class XplgContainerDiagramRestService {
    private data: any[] = [];
    private node = {
        Name: '',
        ReportingPerson: 'Elizabeth',
        Role: 'Lead'
    };

    embeddedDiagramAction(diagramData: any, diagramType: string): Observable<CommunicationEvent> {
        const isAsyncMode = diagramData?.data?.edges.some(edge => edge.styleId === 'edge_twoWay');
        const diagramNodes = diagramData?.data?.nodes.map((nodeObj, index, arr) => {
            const edges = diagramData.data.edges;
            const edge = edges.find(edge => edge.id === nodeObj.incomingEdgeIds[0]);
            const reporter = edge?.sourceId;
            const actionConf = nodeObj.data.find(data => data.type === 'triggers');
            const position = DiagramService.connectorPosition(nodeObj, edges, index, arr);

            return {
                id: nodeObj.id,
                position: position,
                value: nodeObj.data[0].value,
                time: nodeObj.data[2].value.join(),
                parent: reporter,
                amount: nodeObj.data[1].value,
                layout: 'narrow',
                uiPrefs: nodeObj.uiPrefs,
                rest: actionConf?.value?.onClick
            };
        });

        return of({
            type: 'injectData',
            data: {
                diagramType,
                dataType: 'full',
                async: isAsyncMode,
                data: diagramNodes || this.data
            },
            theme: 'default'
        });
    }

    addNode(): Observable<CommunicationEvent> {
        this.node = {...this.node, Name: Date.now().toString()};
        this.data = [...this.data, this.node];
        return of({
            type: 'injectData',
            data: {
                dataType: 'full',
                data: this.data
            },
            theme: 'default'
        });
    }

    removeNode(): Observable<CommunicationEvent> {
        return of({
            type: 'injectData',
            data: {
                dataType: 'full',
                data: this.data.slice(0, this.data.length)
            },
            theme: 'default'
        });
    }
}
