import {Injectable} from '@angular/core';
import {
    ConnectorConstraints,
    ConnectorModel,
    Diagram,
    NodeConstraints,
    NodeModel
} from '@syncfusion/ej2-angular-diagrams';
import {NodeInfo} from '../syncfusion-impl.component';

@Injectable()
export class AdvancedLayoutAsyncService {

    static nodeDefault(node: NodeModel & {data: NodeInfo}): NodeModel {
        const value = node.data?.value;
        const time = node.data?.time;
        const ports = node.data?.ports;
        const icon = node.data?.uiPrefs?.icon || 'gi-host-icon';
        const uiPrefs = node.data?.uiPrefs;
        const hiddenNode = uiPrefs?.hidden;
        let hiddenClass = '';
        let amount = node.data.amount;
        if (hiddenNode) {
            hiddenClass = 'hidden-node';
        }
        node.height = 115;
        node.minWidth = 60;
        node.shape = {
            content: `<div class="advanced-node-container ${hiddenClass}">
                    <div class="node-data" style="border-color: ${uiPrefs?.color}">
                        <div class="node-icon">
                            <span class="icon gi ${icon}" style="color: ${uiPrefs?.color}">
                                <span class="path1"></span>
                                <span class="path2"></span>
                            </span>
                            <i class="icon-border gi gi-border-node-icon" style="color: ${uiPrefs?.color}"></i>
                        </div>
                        <div class="node-container__value xplg-label-strong"
                             style="background-color: ${uiPrefs?.color};
                             color: ${uiPrefs?.color}">
                            <span class="node-value text-ellipsis">${value}</span>
                            ${amount ? `<span class="node-amount fw-regular">${amount}</span>` : ''}
                        </div>
                        <div class="node-container__timestamp xplg-underline text-ellipsis"
                            style="color: ${uiPrefs?.color}">${time}
                        </div>
                    </div>
                </div>`,
            type: 'HTML'
        };
        if (uiPrefs?.hidden) {
            node.height = 150;
            node.minWidth = 1;
            node.width = 1;
            node.style = {opacity: 0};
            node.constraints = NodeConstraints.None;
        }
        node.ports = ports;
        return node;
    }

    static connectorDefaults(connector: ConnectorModel, diagram?: Diagram): ConnectorModel {
        const sourceNode = diagram?.nodes.find(node => node.id === connector.sourceID);
        const targetNode = diagram?.nodes.find(node => node.id === connector.targetID);
        const sourceEdgeColor = (sourceNode?.data as NodeInfo)?.uiPrefs?.color;
        const targetEdgeColor = (targetNode?.data as NodeInfo)?.uiPrefs?.color;
        const edgeColor = sourceEdgeColor || targetEdgeColor;
        const hiddenNode = sourceNode?.style?.opacity;
        const sourcePosition = (sourceNode as any).properties.data.position;
        const targetPosition = (targetNode as any).properties.data.position;
        connector.type = "Orthogonal";
        connector.cornerRadius = 10;
        connector.style = {
            strokeDashArray: '3,4',
            strokeWidth: 3,
            strokeColor: edgeColor,
            fill: edgeColor
        };
        connector.targetDecorator = {
            style: {
                fill: edgeColor,
                strokeColor: edgeColor
            }
        };
        connector.sourceDecorator = {
            style: {
                fill: edgeColor,
                strokeColor: edgeColor
            }
        };
        connector.constraints = ConnectorConstraints.Default & ~ConnectorConstraints.Drag & ~ConnectorConstraints.Select;
        if (hiddenNode) {
            connector.sourcePortID = 'asyncSourcePort';
            connector.targetPortID = 'asyncTargetPortLeft';
            if (sourcePosition === 'left' || sourcePosition === null) {
                connector.style = {
                    opacity: 0
                };
                connector.targetDecorator = {
                    style: {
                        opacity: 0
                    }
                };
            }
        } else {
            connector.sourcePortID = 'asyncTargetPortRight';
            connector.targetPortID = 'asyncSourcePort';
            connector.sourceDecorator = {shape: 'OutdentedArrow'};
            connector.targetDecorator = {shape: 'None'};
            if (targetPosition === 'right' || targetPosition === null) {
                connector.style = {
                    opacity: 0
                };
                connector.targetDecorator = {
                    style: {
                        opacity: 0
                    }
                };
                connector.sourceDecorator = {
                    style: {
                        opacity: 0
                    }
                };
            }
        }
        return connector;
    }
}
