import {Injectable} from '@angular/core';
import {ConnectorConstraints, ConnectorModel, Diagram, NodeModel} from "@syncfusion/ej2-angular-diagrams";
import {NodeInfo} from "../syncfusion-impl.component";

@Injectable()
export class AdvancedLayoutService {

    static nodeDefault(node: NodeModel & {data: NodeInfo}): NodeModel {
        const value = node.data?.value;
        const time = node.data?.time;
        const uiPrefs= node.data?.uiPrefs;
        const icon = uiPrefs?.icon || 'gi-host-icon';
        let amount = node.data?.amount;
        node.height = 115;
        node.shape = {
            content: `<div class="advanced-node-container" data-connector-color="${uiPrefs?.color}">
                  <div class="node-data" style="border-color: ${uiPrefs?.color}">
                    <div class="node-icon">
                        <span class="icon gi ${icon}" style="color: ${uiPrefs?.color}">
                            <span class="path1"></span>
                            <span class="path2"></span>
                        </span>
                        <i class="icon-border gi gi-border-node-icon" style="color: ${uiPrefs?.color}"></i>
                    </div>
                    <div class="node-container__value xplg-label-strong"
                         style="background-color: ${uiPrefs?.color};
                         color: ${uiPrefs?.color}">
                        <span class="node-value text-ellipsis">${value}</span>
                        ${amount ? `<span class="node-amount fw-regular">${amount}</span>` : ''}
                    </div>
                    <div class="node-container__timestamp xplg-underline text-ellipsis"
                        style="color: ${uiPrefs?.color}">${time}
                    </div>
                  </div>
                </div>`,
            type: 'HTML'
        };
        return node;
    }

    static connectorDefaults(connector: ConnectorModel, diagram?: Diagram): ConnectorModel {
        const sourceNode = diagram?.nodes.find(node => node.id === connector.sourceID);
        const targetNode = diagram?.nodes.find(node => node.id === connector.targetID);
        const sourceEdgeColor = (sourceNode?.data as NodeInfo)?.uiPrefs?.color;
        const targetEdgeColor = (targetNode?.data as NodeInfo)?.uiPrefs?.color;
        const edgeColor = sourceEdgeColor || targetEdgeColor;
        connector.type = 'Orthogonal';
        connector.cornerRadius = 7;
        connector.constraints = ConnectorConstraints.Default & ~ConnectorConstraints.Drag & ~ConnectorConstraints.Select;
        connector.style = {
            strokeDashArray: '3,4',
            strokeWidth: 3,
            strokeColor: edgeColor,
            fill: edgeColor
        };
        connector.targetDecorator = {
            style: {
                fill: edgeColor,
                strokeColor: edgeColor
            }
        }
        connector.constraints = ConnectorConstraints.Default & ~ConnectorConstraints.Drag;
        return connector;
    }
}
