import {Injectable} from '@angular/core';
import {ConnectorModel, NodeModel} from '@syncfusion/ej2-angular-diagrams';
import {NodeInfo} from '../syncfusion-impl.component';
import {IDataTransformer} from '../idatatransformer';
import {DataTransformerAbstract} from './data-transformer.abstract';

@Injectable()
export class NarrowLayoutService {

    constructor() {
    }
    
    static nodeDefault(node: NodeModel & {data: NodeInfo}): NodeModel {
        let value = node.data?.value;
        const amount = node.data?.amount;
        const uiPrefs = node.data?.uiPrefs;
        const time = node.data?.time;
        if (value?.length > 10) {
            let lastSegmentStartPosition = 0;
            const defaultStartPosition = 6;
            const amountLength = amount.length + 2;
            if ((defaultStartPosition - amountLength) > 0) {
                lastSegmentStartPosition = defaultStartPosition - amountLength;
            }

            value = `${value.slice(0, 3)}...${value.slice(value.length - lastSegmentStartPosition, value.length)}`;
        }
        node.height = 60;
        node.minWidth = 36;
        node.maxWidth = 98;
        node.width = 98;
        node.ports = [
            {
                verticalAlignment: 'Bottom',
                horizontalAlignment: 'Right',
                offset: {
                    x: 10,
                    y: 1
                },
                shape: 'Circle'
            },
            {
                verticalAlignment: 'Bottom',
                horizontalAlignment: 'Right',
                offset: {
                    x: 10,
                    y: 1
                }
            }
        ];

        node.shape = {
            content:`<div class="node-container">
                        <div class="node-container__timestamp xplg-overline fw-medium text-ellipsis"
                            style="color: ${uiPrefs?.color}">${time}</div>
                        <div class="node-container__value xplg-overline" style="border-left-color: ${uiPrefs?.iconBackground}">
                            <span class="node-value">${value}</span>
                            <span class="node-amount fw-regular">(${amount})</span>
                        </div>
                    </div>`,
            type:'HTML'
        };
        return node;
    }

    static connectorDefaults(connector: ConnectorModel): ConnectorModel {
        connector.type = "Orthogonal";
        connector.cornerRadius = 7;
        connector.targetPadding = 100;
        connector.style = {
            strokeDashArray: '2,4',
            strokeWidth: 1,
            strokeColor: 'var(--diagram-legend-other)',
            fill: 'var(--diagram-legend-other)'
        }
        return connector;
    }
}
