import {ConnectorModel, Diagram, IClickEventArgs, IDropEventArgs, NodeModel} from '@syncfusion/ej2-angular-diagrams';
import {IXplgNode, NodeInfo} from './syncfusion-impl.component';

/**
 * Syncfusion Diagram component interface
 */
export interface IEmbeddedSyncfusionComponent extends IEmbeddedDiagramComponent {

    /**
     * Syncfusion Node transformer (Syncfusion Transformer)
     */
    nodeDefaults: (node: NodeModel & {data: NodeInfo}) => NodeModel;

    /**
     * Syncfusion Edge transformer
     */
    connectorDefaults: (connector: ConnectorModel, diagram?: Diagram) => ConnectorModel;

}

/**
 * Common
 */
interface IEmbeddedDiagramComponent {
    /**
     * Initialize Diagram with provided data
     */
    bootstrapDiagram: (data: any) => void;

    /**
     * Handle node clickIXplgNode
     */
    onClick: (event: IClickEventArgs) => void;

    /**
     * Handle node drop
     */
    onDrop: (event: IDropEventArgs) => void;

    /**
     * Handle node  double click
     */
    onDblClick: (event: IClickEventArgs) => void;

    /**
     * Add node (request data for a new node)
     */
    addNode: () => void;

    /**
     * Remove node
     */
    removeNode: () => void;

    /**
     * XplgNode transformer (Common transformer)
     */
    nodeTransformer: (node: IXplgNode) => IXplgNode;
}
