import {Injectable} from '@angular/core';
import {NodeModel} from "@syncfusion/ej2-angular-diagrams";
import {NodeInfo} from "../syncfusion-impl.component";

@Injectable()
export class NodeUtilsService {

    private canvas: HTMLCanvasElement | null = null;

    calculateNodeWidth(nodeModel: NodeModel): number {
        const nodeTextStyle = 'bold 18px Roboto';
        const nodePadding = 16;
        const nodeMargin = 8;
        const amountMargin = 5;
        const nodeValueWidth = Math.ceil(this.calculateTextWidth((nodeModel.data as NodeInfo).value, nodeTextStyle));
        let nodeAmountWidth = Math.ceil(this.calculateTextWidth((nodeModel.data as NodeInfo).amount, nodeTextStyle));
        if (nodeAmountWidth > 0) {
            nodeAmountWidth += amountMargin;
        }
        return nodeValueWidth + nodeAmountWidth + nodePadding + nodeMargin;
    }

    private calculateTextWidth(text: string, font: string): number {
        if (!this.canvas) {
            this.canvas = document.createElement('canvas');
        }

        const context = this.canvas.getContext('2d');
        if (context) {
            context.font = font;
            const metrics = context.measureText(text);
            return metrics.width + 2;
        }
        return -1;
    }
}
