class SSEClient {
    constructor(endpointUrl) {
        this.endpointUrl = endpointUrl;
        this.subscriberId = new Date().getTime();
        this.endpointUrl = this.endpointUrl + "?subscriberId=" + this.subscriberId;
        this.sseData = document.getElementById('sse-data');
        this.eventSource = null;

        // Get references to the buttons
        this.connectButton = document.getElementById('connectButton');
        this.disconnectButton = document.getElementById('disconnectButton');
        this.sendButton = document.getElementById('sendMessage');
        this.sendBroadcastButton = document.getElementById('sendBroadcastMessage');

        // Add event listeners to the buttons
        this.connectButton.addEventListener('click', () => this.connect());
        this.disconnectButton.addEventListener('click', () => this.disconnect());
        this.sendButton.addEventListener('click', () => this.sendMessage(false));
        this.sendBroadcastButton.addEventListener('click', () => this.sendMessage(true));
    }

    connect() {
        this.eventSource = new EventSource(this.endpointUrl,{ retry: false });

        this.eventSource.addEventListener('open', () => {
            console.log('SSE connection opened.');
            // Disable the "Connect" button and enable the "Disconnect" button
            this.connectButton.disabled = true;
            this.disconnectButton.disabled = false;
        });

        this.eventSource.addEventListener('error', (error) => {
            console.error('SSE error:', error);
        });

        this.eventSource.onmessage = (event) => {
            var msg = "";
            if (event.name){
                msg += "name:" + event.name + " , ";
            }
            if (event.type){
                msg += "type:" + event.type + " , ";
            }


            const eventData = JSON.parse(event.data);
            this.displayEventData(msg + "eventData:" +  JSON.stringify(eventData.response));
        };
    }

    displayEventData(data) {
        const eventItem = document.createElement('div');
        eventItem.textContent = data;
        this.sseData.appendChild(eventItem);
    }

    disconnect() {
        if (this.eventSource) {
            this.eventSource.close();
            console.log('SSE connection closed.');
            // Enable the "Connect" button and disable the "Disconnect" button
            this.connectButton.disabled = false;
            this.disconnectButton.disabled = true;
        }
    }

     sendMessage(broadcast) {
        var text = document.getElementById('message');
        console.log("text = " + text.value);

        var parameters = "message=" + encodeURIComponent(text.value);
        var restName = "send-message";
        if (broadcast){
            restName = "send-broadcast-message";
        }
        const url = 'http://localhost:30303/logeye/rest/test-sse/' + restName + '?' + parameters;
        var token = localStorage.getItem('xplg-token');
        var subscriberId = this.subscriberId;

        const headers = {
          'Authorization': 'Bearer ' + token,
          'subscriberId': subscriberId
        };

        const requestData = {
          message: text
        };

        // Define the request configuration including headers
        const requestOptions = {
          method: 'POST', // HTTP method (e.g., GET, POST, PUT, DELETE)
          headers: headers, // Include your custom headers here
          body: JSON.stringify(requestData)
        };

        // Send a GET request
        fetch(url,requestOptions)
          .then(response => {
            if (!response.ok) {
              throw new Error('Network response was not ok');
            }
            return response.json(); // Parse the response as JSON
          })
          .then(data => {
            // Handle the data from the response
            console.log(data);
          })
          .catch(error => {
            // Handle any errors that occurred during the fetch
            console.error('Fetch error:', error);
          });
    }
}


// Usage:
const sse = new SSEClient('/logeye/rest/sse'); // Replace with your SSE server endpoint
