(function () {
    if (typeof _ !== 'undefined' && _.mixin) {
        _.mixin({
            sortByNat: function (reverseSort, obj, value, context) {
                var iterator = _.isFunction(value) ? value : function (obj) {
                    return obj[value];
                };
                return _.map(_.map(obj, function (value, index, list) {
                    return {
                        value: value,
                        index: index,
                        criteria: iterator.call(context, value, index, list)
                    };
                }).sort(function (left, right) {
                    var a = left.criteria,
                        b = right.criteria,
                        value;

                    if (typeof a == 'string') a = a.toLowerCase();
                    if (typeof b == 'string') b = b.toLowerCase();

                    if (reverseSort) {
                        value = naturalSort(b, a);
                        if (value === 0) {
                            return naturalSort(right.index, left.index);
                        }
                        return value;
                    } else {
                        value = naturalSort(a, b);
                        if (value === 0) {
                            return naturalSort(right.index, left.index);
                        }
                        return value;
                    }
                }), 'value');
            }
        });

        function naturalSort(a, b) {
            var re = /(^-?[0-9]+(\.?[0-9]*)[df]?e?[0-9]?$|^0x[0-9a-f]+$|[0-9]+)/gi,
                sre = /(^[ ]*|[ ]*$)/g,
                dre = /(^([\w ]+,?[\w ]+)?[\w ]+,?[\w ]+\d+:\d+(:\d+)?[\w ]?|^\d{1,4}[\/\-]\d{1,4}[\/\-]\d{1,4}|^\w+, \w+ \d+, \d{4})/,
                hre = /^0x[0-9a-f]+$/i,
                ore = /^0/,
                com = /,/g,
                i = function (s) {
                    return naturalSort.insensitive && ('' + s).toLowerCase() || '' + s
                },

                x = i(a).replace(sre, '').replace(com, '') || '',
                y = i(b).replace(sre, '').replace(com, '') || '',

                xN = x.replace(re, '\0$1\0').replace(/\0$/, '').replace(/^\0/, '').split('\0'),
                yN = y.replace(re, '\0$1\0').replace(/\0$/, '').replace(/^\0/, '').split('\0'),

                xD = parseInt(x.match(hre)) || (xN.length != 1 && x.match(dre) && Date.parse(x)),
                yD = parseInt(y.match(hre)) || xD && y.match(dre) && Date.parse(y) || null,
                oFxNcL, oFyNcL;

            if (yD)
                if (xD < yD) return -1;
                else if (xD > yD) return 1;

            for (var cLoc = 0, numS = Math.max(xN.length, yN.length); cLoc < numS; cLoc++) {

                oFxNcL = !(xN[cLoc] || '').match(ore) && parseFloat(xN[cLoc]) || xN[cLoc] || 0;
                oFyNcL = !(yN[cLoc] || '').match(ore) && parseFloat(yN[cLoc]) || yN[cLoc] || 0;

                if (isNaN(oFxNcL) !== isNaN(oFyNcL)) {
                    return (isNaN(oFxNcL)) ? 1 : -1;
                }

                else if (typeof oFxNcL !== typeof oFyNcL) {
                    oFxNcL += '';
                    oFyNcL += '';
                }
                if (oFxNcL < oFyNcL) return -1;
                if (oFxNcL > oFyNcL) return 1;
            }
            return 0;
        }
    }
}());
