function stackedGroupedChart(HC) {

    /* Global flags */

    var PHANTOM_MODE = false,
        STACK_INDEX = 0;

    var merge = HC.merge,
        each = HC.each;

    HC.wrap(HC.Legend.prototype, 'getAllItems', function (proceed) {
        var allItems = [],
            colorAxis = this.chart.colorAxis[0],
            legendID = this.options.legendID;

        if (colorAxis && colorAxis.options) {
            if (colorAxis.options.showInLegend) {
                // Data classes
                if (colorAxis.options.dataClasses) {
                    allItems = allItems.concat(colorAxis.getDataClassLegendSymbols());
                    // Gradient legend
                } else {
                    // Add this axis on top
                    allItems.push(colorAxis);
                }
            }

            // Don't add the color axis' series
            each(colorAxis.series, function (series) {
                series.options.showInLegend = false;
            });
        }

        if (!_.isUndefined(this.chart.options.legendTop) && !_.isUndefined(this.chart.options.legendBottom)) {
            each(this.chart.series, function (series) {
                if (series) {
                    var seriesOptions = series.options;
                    if (!isNaN(legendID) && (seriesOptions.legendID === legendID)) {
                        allItems = allItems.concat(
                            series.legendItems ||
                            (seriesOptions.legendType === 'point' ?
                                series.data :
                                series)
                        );
                    }
                }
            });
            return allItems;
        }

        return allItems.concat(proceed.call(this));
    });

    // 1.1 WRAP METHODS
    // Double legend extension:

    // HC.wrap(HC.Chart.prototype, 'render', function (p) {
    //     var chart = this,
    //         stackedGroupedColumn = HC.stackedGroupedColumn,
    //         chartOptions = chart.options;
    //
    //     chart.titleOffset = [1, 1, 1, 1];
    //     if (!_.isUndefined(chartOptions.legendTop) && !_.isUndefined(chartOptions.legendBottom)) {
    //         chart.topLegend = new HC.Legend(chart, merge(chartOptions.legend, chartOptions.legendTop, {
    //             legendID: 0
    //         }));
    //
    //         chart.botLegend = new HC.Legend(chart, merge(chartOptions.legend, chartOptions.legendBottom, {
    //             legendID: 1
    //         }));
    //
    //         stackedGroupedColumn.renderLegendElements.call(chart);
    //     }
    //
    //     p.call(chart);
    //
    //     if (PHANTOM_MODE) {
    //         stackedGroupedColumn.addLabelsBackgrounds.call(chart);
    //         stackedGroupedColumn.addStackLabelsBackgrounds.call(chart, chartOptions, stackedGroupedColumn.utils.getStackIndex());
    //     }
    //
    // });

    //old version

    HC.wrap(HC.Chart.prototype, 'getMargins', function (p, skip) {
        var chart = this;
        if (!_.isUndefined(chart.topLegend) && !_.isUndefined(chart.botLegend)) {
            chart.extraMargin = {
                type: 'marginBottom',
                value: chart.topLegend.legendHeight + chart.botLegend.legendHeight + 25
            };
        }
        p.call(chart, skip);
    });

    HC.wrap(HC.Chart.prototype, 'redraw', function (p, r, a) {
        var chart = this,
            stackedGroupedColumn = HC.stackedGroupedColumn;

        p.call(chart, r, a);

        if (!_.isUndefined(chart.topLegend) && !_.isUndefined(chart.botLegend)) {
            chart.topLegend.render();
            chart.botLegend.render();
            stackedGroupedColumn.renderLegendElements.call(this);
        }

    });

    // 1.2 CUSTOM METHODS
    HC.stackedGroupedColumn = {
        renderLegendElements: function () {
            var chart = this,
                r = chart.renderer,
                topLegend = chart.topLegend,
                topLegendOptions = topLegend.options,
                fontSize = 4,
                bottomTitle;

            // render numbers
            if (topLegend.allItems) {
                each(topLegend.allItems, function (item, i) {
                    if (item.legendSymbol) {
                        item.legendSymbol.translate(0, 5);

                        if (i > 8) {
                            fontSize = 0;
                        }

                        if (item.background) {
                            item.background.destroy();
                            item.background = null;
                        }

                        item.background = r.text(i + 1, fontSize, (topLegendOptions.lineHeight / 2) + (topLegend.padding / 2) + 2)
                            .attr({
                                'zIndex': 10,
                                'font-weight': 'bold',
                                'font-size': '12px'
                            })
                            .add(item.legendGroup);
                    }
                });
            }

            // render background on the second legend
            if (chart.botLegend && chart.botLegend.title) {
                bottomTitle = chart.botLegend.title.getBBox();

                chart.botLegend.box.attr({
                    height: bottomTitle.height + chart.botLegend.padding,
                    y: bottomTitle.height + chart.botLegend.padding
                });
            }
        },
        parseData: function (chartHolder) {

            var stackIndex = 0,
                series = [],
                stackSeries = [],
                legendName,
                legendID,
                id,
                map = {},
                colors = {},
                colors2 = {};

            each(chartHolder.series, function (seriesItem) {
                if (seriesItem.data && seriesItem.data.length > 0) {
                    var arr = seriesItem.id.split(',');
                    id = arr[0].trim();
                    var stackId = arr[1].trim();
                    var ob = {};
                    if (_.isUndefined(map[id])) {
                        stackIndex++;

                        ob = map[id] = {
                            stackIndex: stackIndex
                        };
                    } else {
                        ob = map[id];
                    }
                    var bColor = colors[id];
                    var sColor = colors2[stackId];


                    // fix colors
                    each(seriesItem.data, function (point) {
                        point.color = sColor;
                    });


                    // set borders and stacks
                    series.push(_.extend(seriesItem, {
                        stack: ob.stackIndex - 1,
                        borderCustomColor: bColor,
                        borderWidth: 2,
                        borderColor: bColor
                    }));
                } else {
                    legendName = seriesItem.name;
                    legendID = seriesItem.legendID;
                    if (!_.isUndefined(seriesItem.color)) {
                        if (legendID == 0) {
                            colors[seriesItem.id] = seriesItem.color;
                        } else {
                            colors2[seriesItem.id] = seriesItem.color;
                        }
                    }

                    series.push(_.extend(seriesItem, {
                        legendName: legendName,
                        legendID: legendID
                    }));
                }
            });

            return {
                series: series.concat(stackSeries),
                stackIndex: stackIndex
            };
        },
        stackedTooltip: function () {
            var point = this,
                points = [],
                machineID = point.series.options.id.split(',')[0].trim(),
                series = point.series.chart.series,
                currentFormatStr = point.series.xAxis.userOptions.labels.format,
                currentFormatValue = currentFormatStr.substring(currentFormatStr.indexOf(':') + 1, currentFormatStr.length - 1).trim(),
                txt = '<span style="font-weight: 600">' + HC.dateFormat(currentFormatValue, point.key) + '</span><br/>' + machineID + '<br/>',
                id;

            each(series, function (seriesItem) {
                id = seriesItem.options.id.split(',');
                if (id[0].trim() === machineID) {
                    var splittedPart = (id.length > 1) ? 1 : 0;
                    each(seriesItem.data, function (p) {
                        if (p.x === point.x) {
                            p.splittedPart = splittedPart;
                            points.push(p);
                        }
                    });
                }
            });

            points.sort(function (a, b) {
                return b.y - a.y;
            });

            points.forEach(function (p) {
                txt += '<span style="color:' + p.color + '">' + id[p.splittedPart].trim() + '</span> : ' + HC.numberFormat(p.y, 3, '.', '.') + '<br/>';
            });

            return txt;
        },
        stackedYLabels: function () {
            return this.stack + 1;
        },
        addLabelsBackgrounds: function () {
            var chart = this,
                yAxis = chart.yAxis[0],
                xAxis = chart.xAxis[0],
                yAxisOptions = yAxis.options,
                stackedGroupedColumn = HC.stackedGroupedColumn,
                yAxisVisible = stackedGroupedColumn.utils.isAxisVisible(yAxis),
                ticks = yAxis.ticks,
                offset = yAxisOptions.offset || 0,
                showFirstLabel = yAxisOptions.showFirstLabel,
                yAxisLabelsStyle = yAxisOptions.labels && yAxisOptions.labels.style,
                backgroundColor = yAxisLabelsStyle && yAxisLabelsStyle.backgroundColor,
                r = chart.renderer,
                gridLine,
                xAxisLine,
                path,
                width,
                label,
                bbox;

            for (var tick in ticks) {

                // fix grid lines width
                gridLine = ticks[tick].gridLine;

                if (gridLine) {
                    path = gridLine.d.split(' ');

                    path[1] = yAxisOptions.offset - 5 || 0;

                    gridLine.attr({
                        d: path.join(' ')
                    });
                }

                label = ticks[tick].label;

                if (label && (ticks[tick].isFirst === showFirstLabel)) {
                    bbox = label.getBBox();

                    width = yAxisLabelsStyle.width || bbox.width + 10;

                    if (ticks[tick].background) { // redraw
                        ticks[tick].background.destroy();
                        ticks[tick].background = null;
                    }

                    if (yAxisVisible) {
                        ticks[tick].background = r.rect(offset - 5, label.xy.y - bbox.height - 1, width, bbox.height + 10)
                            .attr({
                                fill: backgroundColor
                            })
                            .add();
                    }
                }
            }

            // fix xAxis line width
            xAxisLine = xAxis.axisLine;

            if (xAxisLine) {
                path = xAxisLine.d.split(' ');

                path[1] = yAxisOptions.offset - 5 || 0;

                xAxisLine.attr({
                    d: path.join(' ')
                });
            }
        },
        addStackLabelsBackgrounds: function (config, stackIndex) {
            var chart = this,
                series = chart.series,
                stackedGroupedColumn = HC.stackedGroupedColumn,
                yAxis = chart.yAxis[0],
                stacks = yAxis.stacks,
                r = chart.renderer,
                xAxis,
                yAxisVisible,
                borderWidth,
                shape,
                shapeWidth,
                height,
                labelTextWidth,
                label,
                stack,
                x,
                y;

            each(series, function (seriesItem) {
                each(seriesItem.data, function (point) {

                    if (point.stackLabelBck) {
                        point.stackLabelBck.destroy();
                        point.stackLabelBck = null;
                    }

                    yAxis = point.series.yAxis;
                    if (yAxis) {
                        yAxisVisible = stackedGroupedColumn.utils.isAxisVisible(yAxis);

                        if (yAxisVisible) {
                            shape = point.shapeArgs;
                            xAxis = point.series.xAxis;
                            borderWidth = 2;
                            y = yAxis.toPixels(point.total);
                            height = yAxis.toPixels(0) - y;
                            if (shape && shape.width > 0) { // visible point
                                x = point.plotX + chart.plotLeft + seriesItem.pointXOffset - ((borderWidth + 1) / 2);
                                shapeWidth = shape.width + (2 * borderWidth) - 2;
                                point.stackLabelBck = r.rect(x, y, shapeWidth, height + 20)
                                    .attr({
                                        zIndex: 1,
                                        fill: seriesItem.options.borderCustomColor
                                    })
                                    .add();
                            }
                        }
                    }
                });
            });

            // move labels down
            for (var key in stacks) {
                stack = stacks[key];

                for (var timestamp in stack) {
                    label = stack[timestamp].label;

                    if (label) {
                        labelTextWidth = label.element.getBoundingClientRect().width || 0;
                        label.attr('y', label.y + 22);
                        if (!PHANTOM_MODE) {
                            height = label.getBBox().height;
                        }
                    }
                }
            }
        },
        legendItemClick: function () {
            var series = this,
                chart = series.chart,
                allSeries = chart.series,
                legend = chart.legend,
                visible = series.visible,
                name = series.options.id;

            each(allSeries, function (seriesItem) {


                var found = false;
                if (seriesItem.options.id.indexOf(name + " ,") == 0) {
                    found = true;
                } else {
                    var index = seriesItem.options.id.indexOf(", " + name);
                    if (index + 2 + name.length == seriesItem.options.id.length) {
                        found = true;
                    }
                }
                if (found) {
                    each(seriesItem.data, function (p) {
                        var eraseOrHide = (series.userOptions.legendID) ? 0 : null;
                        p.update({
                            oldY: p.y + 22,
                            y: visible ? eraseOrHide + 22 : p.oldY + 22,
                        }, false);
                    });
                }
            });

            chart.redraw();

            legend.colorizeItem(series, !visible);
            series.visible = !visible;

            return false;
        },

        getContainerWidth: function (stackIndex, tickPositions, config) {

            var containerWidth = config.chart.width,
                seriesPlotOptions = config.plotOptions && config.plotOptions.series,
                minPointWidth = seriesPlotOptions && seriesPlotOptions.minPointWidth,
                maxPointWidth = (containerWidth / 3),
                seriesLength = (config.plotOptions && config.plotOptions.series && config.plotOptions.series.length) || 1,
                ticksLength = (tickPositions && tickPositions.length) || seriesLength,
                chartWidth = containerWidth - config.chart.marginLeft,
                pointWidth;

            pointWidth = 4 * containerWidth / ((5 * stackIndex * ticksLength) + (4 * ticksLength));

            if (pointWidth >= maxPointWidth) {
                pointWidth = maxPointWidth;

            } else if (pointWidth < minPointWidth) {
                chartWidth = (20 * stackIndex * ticksLength) + (16 * ticksLength);
                pointWidth = minPointWidth;
            }

            return {
                chartWidth: chartWidth,
                pointWidth: pointWidth,
                groupPadding: (stackIndex === 1) ? 0.5 : null,
                pointPlacement: HC.stackedGroupedColumn.utils.getPointPlacemnt(stackIndex)
            };
        },

        utils: {
            isAxisVisible: function (axis) {
                if (axis.dataMin === null && axis.dataMax === null) {
                    return false;
                }
                return true;
            },
            getPointPlacemnt: function (stackIndex) {
                return (stackIndex > 1) ? -(1 / stackIndex) / 3 : null;
            },
            setPhantomMode: function (mode) {
                PHANTOM_MODE = mode;
            },
            setStackIndex: function (index) {
                STACK_INDEX = index;
                return STACK_INDEX;
            },
            getStackIndex: function () {
                return STACK_INDEX;
            }
        }
    };
}
