<%@ page import="java.net.*" %>
<%@ page import="javax.net.ssl.*" %>
<%@ page import="java.net.http.*" %>
<%@ page import="java.io.*" %>
<%@ page import="java.util.*" %>
<%@ page import="java.security.*" %>
<%@ page import="java.util.logging.*" %>
<%@ page import="java.security.cert.*" %>
<%@ page import="org.json.simple.*" %>
<%@ page import="java.time.*" %>
<%@ page import="java.nio.charset.*" %>

<%@ page import="org.bouncycastle.asn1.x500.X500Name" %>
<%@ page import="org.bouncycastle.cert.X509v3CertificateBuilder" %>
<%@ page import="org.bouncycastle.cert.jcajce.*" %>
<%@ page import="org.bouncycastle.jce.provider.BouncyCastleProvider" %>
<%@ page import="org.bouncycastle.operator.ContentSigner" %>
<%@ page import="org.bouncycastle.operator.jcajce.JcaContentSignerBuilder" %>
<%@ page import="org.bouncycastle.util.io.pem.*" %>
<%@ page import="org.opensaml.core.config.InitializationService" %>
<%@ page import="org.opensaml.security.credential.*" %>
<%@ page import="java.math.*" %>
<%@ page import="java.nio.file.*" %>
<%@ page import="java.security.*" %>
<%@ page import="java.security.cert.*" %>
<%@ page import="java.security.spec.PKCS8EncodedKeySpec" %>
<%@ page import="org.opensaml.core.config.InitializationService" %>
<%@ page import="org.opensaml.core.xml.config.XMLObjectProviderRegistrySupport" %>
<%@ page import="org.opensaml.saml.saml2.core.AuthnRequest" %>
<%@ page import="org.w3c.dom.*" %>
<%@ page import="javax.xml.transform.*" %>
<%@ page import="javax.xml.transform.dom.DOMSource" %>
<%@ page import="javax.xml.transform.stream.StreamResult" %>
<%@ page import="javax.xml.parsers.*" %>
<%@ page import="io.jsonwebtoken.*" %>
<%@ page import="io.jsonwebtoken.security.*" %>




<%!

class OAuth2Common {

    protected Map<String, String> contextParams = new HashMap();
    protected SSOCommon ssoCommon = new SSOCommon();

    public OAuth2Common(){}

    public OAuth2Common(Map<String, String> contextParams){
        this.contextParams = contextParams;
        ssoCommon.setContextParams(contextParams);
    }

    public void setContextParams(Map<String, String> contextParams){
        this.contextParams = contextParams;
        ssoCommon.setContextParams(contextParams);
    }

    public SSOCommon getSSOCommon(){
        return ssoCommon;
    }

    public void initialize() {
        try {
           ssoCommon.initialize();
        }catch (Exception e){
            logError("Failed to initialize saml common",e);
        }

    }

    public String buildAuthUrl() throws Exception {
        String requestBaseUrl = getOrDefault(contextParams,"requestBaseUrl","");
        String callbackUrl = getOrDefault(contextParams,"oauth-callbackURL","");
        if (!callbackUrl.isEmpty()  && !requestBaseUrl.isEmpty() && callbackUrl.startsWith("${requestBaseUrl}")){
            callbackUrl = requestBaseUrl + callbackUrl.substring("${requestBaseUrl}".length());
        }

        String destination = getOrDefault(contextParams,"oauth-authurl","");
        if (destination.isEmpty()) return null;

        StringBuilder builder = new StringBuilder();

        builder.append(destination);
        builder.append("?client_id=").append(URLEncoder.encode(getOrDefault(contextParams,"oauth-clientId",""), StandardCharsets.UTF_8.name()));
        builder.append("&response_type=").append(URLEncoder.encode(getOrDefault(contextParams,"oauth-authResponseType","code"), StandardCharsets.UTF_8));
        builder.append("&redirect_uri=").append(URLEncoder.encode(callbackUrl, StandardCharsets.UTF_8.name()));
        builder.append("&scope=").append(URLEncoder.encode(getOrDefault(contextParams,"oauth-scope","User.Read"), StandardCharsets.UTF_8));
        builder.append("&state=").append(UUID.randomUUID().toString());
        return builder.toString();
    }

    public JSONObject getAccessToken(String authorizationCode) throws Exception {
        String requestBaseUrl = getOrDefault(contextParams,"requestBaseUrl","");
        String callbackUrl = getOrDefault(contextParams,"oauth-callbackURL","");
        if (!callbackUrl.isEmpty()  && !requestBaseUrl.isEmpty() && callbackUrl.startsWith("${requestBaseUrl}")){
            callbackUrl = requestBaseUrl + callbackUrl.substring("${requestBaseUrl}".length());
        }

        String tokenEndpoint = getOrDefault(contextParams,"oauth-tokenurl","");
        String clientId = getOrDefault(contextParams,"oauth-clientId","");
        String clientSecret = decryptField(getOrDefault(contextParams,"oauth-clientSecret",""),true);
        String redirectUri = callbackUrl;
        String grantType = getOrDefault(contextParams,"oauth-grantType","authorization_code");
        String scope = getOrDefault(contextParams,"oauth-scope","User.Read");

        String requestBody = String.format(
                "client_id=%s&scope=%s&code=%s&redirect_uri=%s&grant_type=%s&client_secret=%s",
                URLEncoder.encode(clientId, StandardCharsets.UTF_8),
                URLEncoder.encode(scope, StandardCharsets.UTF_8),
                URLEncoder.encode(authorizationCode, StandardCharsets.UTF_8),
                URLEncoder.encode(redirectUri, StandardCharsets.UTF_8),
                URLEncoder.encode(grantType, StandardCharsets.UTF_8),
                URLEncoder.encode(clientSecret, StandardCharsets.UTF_8)
        );

        HttpClient client = HttpClient.newHttpClient();
        HttpRequest request = HttpRequest.newBuilder()
                .uri(new URI(tokenEndpoint))
                .header("Content-Type", "application/x-www-form-urlencoded")
                .POST(HttpRequest.BodyPublishers.ofString(requestBody))
                .build();

        if (ssoCommon.isDebugEnabled()){
            ssoCommon.ssoLogInfo("Access token request end point = " +  tokenEndpoint +  " , and data = " + requestBody);
        }

        HttpResponse<String> response = client.send(request, HttpResponse.BodyHandlers.ofString());
        String body = response.body();

        if (ssoCommon.isDebugEnabled()){
            ssoCommon.ssoLogInfo("Access token response = " +  body);
        }

		if (body != null && body.startsWith("{")){
			JSONObject result = (JSONObject)new JSONParser().parse(body);
            if (result != null){
                String error = (String)result.get("error");
                if (error != null && error.length() > 0){
                    throw new RuntimeException("OAuth - failed to get access token , error result " + body);
                }
            }
            return result;
		}
		return null;
    }
	
	
	 public JSONObject decodeJWTWithoutVerification(String jwt) {
        try {
            String[] parts = jwt.split("\\.");
            if (parts.length != 3) {
                throw new IllegalArgumentException("Invalid JWT token");
            }

			 String header = new String(Base64.getUrlDecoder().decode(parts[0]));

            // Decode the payload
            String payloadJson = new String(Base64.getUrlDecoder().decode(parts[1]));

            if (ssoCommon.isInfoEnabled()){
                ssoCommon.ssoLogInfo("JWT payload = " + new String(payloadJson));
            }
            if (ssoCommon.isDebugEnabled()){
                ssoCommon.ssoLogInfo("JWT header = " +  header);
            }

            // Parse the JSON payload
           return (JSONObject)new JSONParser().parse(payloadJson);
        } catch (Exception e) {
			e.printStackTrace();
            throw new RuntimeException("Failed to decode JWT", e);
        }
    }
	
	public JSONObject parseJsonFile(String path) {
        try (FileReader reader = new FileReader(path)) {
            // Parse the JSON file
            JSONObject jsonObject = (JSONObject) new JSONParser().parse(reader);

            return jsonObject;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    public JSONObject createLoginDetailsFromJwt(JSONObject formObject,JSONObject jwtToken) throws Exception {
        JSONObject attributesMapping = (JSONObject)formObject.get("mapping.attributes");
        if (attributesMapping == null){
            attributesMapping = new JSONObject();
        }

        String configurationDir = (String)formObject.get("plugin.configurationDir");
        if (configurationDir != null && configurationDir.length() > 0){
            File f = new File(new File(configurationDir),"mapping/attributes.json");
            if (f.exists()){
                JSONObject attributesMappingFromFile = parseJsonFile(f.getAbsolutePath());
                if (attributesMappingFromFile != null){
                    attributesMapping.putAll(attributesMappingFromFile);
                }
            }
        }
        configurationDir = (String)formObject.get("plugin.cachePath");
        if (configurationDir != null && configurationDir.length() > 0){
            File f = new File(new File(configurationDir),"attributes.json");
            if (f.exists()){
                JSONObject attributesMappingFromFile = parseJsonFile(f.getAbsolutePath());
                if (attributesMappingFromFile != null){
                    attributesMapping.putAll(attributesMappingFromFile);
                }
            }

        }
        JSONObject realmMapping = (JSONObject)formObject.get("mapping.realm");
        if (realmMapping == null){
            realmMapping = new JSONObject();
        }
        configurationDir = (String)formObject.get("plugin.configurationDir");
        if (configurationDir != null && configurationDir.length() > 0){
            File f = new File(new File(configurationDir),"mapping/realm.json");
            if (f.exists()){
                JSONObject realmMappingFromFile = parseJsonFile(f.getAbsolutePath());
                if (realmMappingFromFile != null){
                    realmMapping.putAll(realmMappingFromFile);
                }
            }
        }
        configurationDir = (String)formObject.get("plugin.cachePath");
        if (configurationDir != null && configurationDir.length() > 0){
            File f = new File(new File(configurationDir),"realm.json");
            if (f.exists()){
                JSONObject realmMappingFromFile = parseJsonFile(f.getAbsolutePath());
                if (realmMappingFromFile != null){
                    realmMapping.putAll(realmMappingFromFile);
                }
            }

        }
        return createLoginDetails(formObject,jwtToken,attributesMapping,realmMapping);
    }



    public JSONObject createLoginDetails(JSONObject formObject,JSONObject jwtToken,JSONObject attributesMapping,JSONObject realmMapping) throws Exception {
        if (jwtToken == null) return null;
        JSONObject userInfo = null;

        Map<String,String> attributesMap = new HashMap<>();
        if (attributesMapping != null) {
            for (Object o : attributesMapping.keySet()) {
                Object value = attributesMapping.get(o);
                String key = o.toString();
                if (value instanceof JSONArray){
                    JSONArray arr = (JSONArray)value;
                    for (Object val : arr) {
                        attributesMap.put(val.toString(),key);
                    }
                }else if (value instanceof String){
                    attributesMap.put(value.toString(),key);
                }
            }
        }

        String username = null;



        //System.out.println("attributesMap = " + attributesMap);

        JSONObject usersRealm = null;
        JSONObject groupsRealm = null;
        JSONObject policies = null;
        if (realmMapping != null){
            usersRealm = (JSONObject)realmMapping.get("users");
            groupsRealm = (JSONObject)realmMapping.get("groups");
            policies = (JSONObject)realmMapping.get("policies");
        }


        JSONObject loginDetails = new JSONObject();
        userInfo = new JSONObject();
       // userInfo.put("name" , username);

        loginDetails.put("user",userInfo);

        JSONArray groupsArr = null;
        JSONObject context = null;

        for (Object o : jwtToken.keySet()) {
            Object value = jwtToken.get(o);
            String key = o.toString();
            String attType = attributesMap.get(key);
            if (attType != null && value != null){
                if (attType.equals("displayName")) {
                    String displayName = value.toString();
                    if (displayName != null && displayName.length() > 0) {
                        userInfo.put("displayName", displayName);
                    }
                }else if (attType.equals("username")){
                    username = value.toString();
                    if (username != null && username.length() > 0){
                        userInfo.put("name" , username);
                    }
                }else if (attType.equals("groups")){
                    if (context == null){
                        context = new JSONObject();
                    }
                    if (groupsArr == null){
                        groupsArr = new JSONArray();
                        context.put("groups",groupsArr);
                    }

                    if (value instanceof JSONArray){
                        JSONArray arr = (JSONArray)value;
                        for (Object val : arr) {
                            String groupId = val.toString();
                            JSONObject groupObj = new JSONObject();
                            groupObj.put("name",groupId);
                            if (groupsRealm != null){
                                JSONObject obj = (JSONObject)groupsRealm.get(groupId);
                                if (obj != null){
                                    groupObj = obj;
                                }
                            }
                            groupsArr.add(groupObj);
                        }
                    }else if (value instanceof String){
                        String groupId = value.toString();
                        JSONObject groupObj = new JSONObject();
                        groupObj.put("name",groupId);
                        if (groupsRealm != null){
                            JSONObject obj = (JSONObject)groupsRealm.get(groupId);
                            if (obj != null){
                                groupObj = obj;
                            }
                        }
                        groupsArr.add(groupObj);
                    }
                }
            }
        }

        if (username == null) return null;


        if (context != null){
            loginDetails.put("context",context);
        }

        if (usersRealm != null){
            JSONObject userRealm = (JSONObject)usersRealm.get(username);
            if (userRealm != null){
                loginDetails.putAll(userRealm);
            }
        }
        return loginDetails;
    }

}
%>
