<%@ page import="java.net.*" %>
<%@ page import="javax.net.ssl.*" %>
<%@ page import="java.net.http.*" %>
<%@ page import="java.io.*" %>
<%@ page import="java.util.*" %>
<%@ page import="java.util.logging.*" %>
<%@ page import="java.security.cert.*" %>
<%@ page import="org.json.simple.*" %>
<%@ page import="java.time.*" %>
<%@ page import="java.nio.charset.*" %>

<%@ page import="org.bouncycastle.asn1.x500.X500Name" %>
<%@ page import="org.bouncycastle.cert.X509v3CertificateBuilder" %>
<%@ page import="org.bouncycastle.cert.jcajce.*" %>
<%@ page import="org.bouncycastle.jce.provider.BouncyCastleProvider" %>
<%@ page import="org.bouncycastle.operator.ContentSigner" %>
<%@ page import="org.bouncycastle.operator.jcajce.JcaContentSignerBuilder" %>
<%@ page import="org.bouncycastle.util.io.pem.*" %>
<%@ page import="org.opensaml.core.config.InitializationService" %>
<%@ page import="org.opensaml.security.credential.*" %>
<%@ page import="java.math.*" %>
<%@ page import="java.nio.file.*" %>
<%@ page import="java.security.*" %>
<%@ page import="java.security.cert.*" %>
<%@ page import="java.security.spec.PKCS8EncodedKeySpec" %>
<%@ page import="org.opensaml.core.config.InitializationService" %>
<%@ page import="org.opensaml.core.xml.config.XMLObjectProviderRegistrySupport" %>
<%@ page import="org.opensaml.saml.saml2.core.AuthnRequest" %>
<%@ page import="org.w3c.dom.*" %>
<%@ page import="javax.xml.transform.*" %>
<%@ page import="javax.xml.transform.dom.DOMSource" %>
<%@ page import="javax.xml.transform.stream.StreamResult" %>
<%@ page import="javax.xml.parsers.*" %>

<%@ page import="net.shibboleth.utilities.java.support.xml.SerializeSupport" %>
<%@ page import="org.opensaml.core.xml.*" %>
<%@ page import="org.opensaml.core.xml.config.*" %>
<%@ page import="org.opensaml.core.xml.io.*" %>
<%@ page import="org.opensaml.saml.common.*" %>
<%@ page import="org.opensaml.saml.saml2.core.*" %>
<%@ page import="org.opensaml.saml.saml2.core.impl.*" %>
<%@ page import="org.opensaml.security.*" %>
<%@ page import="org.opensaml.security.credential.*" %>
<%@ page import="org.opensaml.xmlsec.*" %>
<%@ page import="org.opensaml.xmlsec.signature.impl.*" %>
<%@ page import="org.opensaml.xmlsec.signature.support.*" %>
<%@ page import="org.opensaml.saml.common.xml.*" %>



<%!

class SamlCommon {

    protected Map<String, String> contextParams = new HashMap();
    protected SSOCommon ssoCommon = new SSOCommon();

    public SamlCommon(){}

    public SamlCommon(Map<String, String> contextParams){
        this.contextParams = contextParams;
        ssoCommon.setContextParams(contextParams);
    }

    public void setContextParams(Map<String, String> contextParams){
        this.contextParams = contextParams;
        ssoCommon.setContextParams(contextParams);
    }

    public SSOCommon getSSOCommon(){
        return ssoCommon;
    }

    public void initialize() {
        try {
           ssoCommon.initialize();
        }catch (Exception e){
            logError("Failed to initialize saml common",e);
        }

    }



     public void samlInitialize() {
        try {
            InitializationService.initialize();
        } catch (Exception e) {
        }
    }


    public Credential loadIdPCredential(String certificatePath) throws Exception {
        // Load the IdP's certificate from a file
        CertificateFactory certFactory = CertificateFactory.getInstance("X.509");
        X509Certificate idpCertificate = (X509Certificate) certFactory.generateCertificate(new ByteArrayInputStream(Files.readAllBytes(Paths.get(certificatePath))));

        // Create a BasicCredential object using the IdP's certificate
        BasicCredential idpCredential = new BasicCredential(idpCertificate.getPublicKey());


        return idpCredential;
    }


    public BasicCredential generateBasicCredential(String path,Map<String, String> contextParams) throws Exception {
        String keyStoreFormat = getOrDefault(contextParams,"saml-spSignatureStoreFormat","pem");
         if (keyStoreFormat.equals("jks")){
            return generateJksBasicCredential(path,contextParams);
         }
         File dir = new File(path).getAbsoluteFile();
         File f = new File(dir,"private-key.pem");

        X509Certificate certificate = null;
        PrivateKey privateKey = null;
        if (!f.exists()){
            Security.addProvider(new BouncyCastleProvider());
            KeyPairGenerator keyGen = KeyPairGenerator.getInstance("RSA");

            keyGen.initialize(Integer.parseInt(getOrDefault(contextParams,"saml-spSignatureKeyLength","2048")));
            KeyPair keyPair = keyGen.generateKeyPair();
            privateKey = keyPair.getPrivate();
            savePrivateKeyToFile(keyPair.getPrivate(),f.getAbsolutePath());
            savePublicKeyToFile(keyPair.getPublic(),new File(dir,"public-key.pem").getAbsolutePath());

            certificate = generateSelfSignedCertificate(keyPair,contextParams);
            saveCertificateToFile(certificate, new File(dir,"certificate.cer").getAbsolutePath());

        }else {
            privateKey = loadPrivateKeyFromFile(f.getAbsolutePath());
            certificate = loadCertificateFromFile(new File(dir,"certificate.cer").getAbsolutePath());
        }

        // Create credentials
        BasicCredential credential = new BasicCredential(certificate.getPublicKey(), privateKey);
        return credential;

    }

     private BasicCredential generateJksBasicCredential(String path,Map<String, String> contextParams) throws Exception {
             File dir = new File(path).getAbsoluteFile();
             File f = new File(dir,"keystore.jks");
            X509Certificate certificate = null;
            PrivateKey privateKey = null;
            String cnName = getOrDefault(contextParams,"saml-certificateCNName","CN=SAML XPLG SSO");
            String password = getOrDefault(contextParams,"saml-keystorePassword",cnName);
            if (!f.exists()){
                Security.addProvider(new BouncyCastleProvider());
                KeyPairGenerator keyGen = KeyPairGenerator.getInstance("RSA");

                keyGen.initialize(Integer.parseInt(getOrDefault(contextParams,"saml-spSignatureKeyLength","2048")));
                KeyPair keyPair = keyGen.generateKeyPair();
                privateKey = keyPair.getPrivate();
                certificate = generateSelfSignedCertificate(keyPair,contextParams);
                saveCertificateToFile(certificate, new File(dir,"certificate.cer").getAbsolutePath());

                KeyStore keyStore = KeyStore.getInstance("JKS");
                keyStore.load(null, null);


                // Set the private key and the certificate in the keystore
                keyStore.setKeyEntry("saml", keyPair.getPrivate(), password.toCharArray(), new java.security.cert.Certificate[]{certificate});

                // Save the keystore to a file
                try (FileOutputStream fos = new FileOutputStream(f)) {
                    keyStore.store(fos, password.toCharArray());
                }

            }else {
                KeyStore keyStore = KeyStore.getInstance("JKS");
                FileInputStream fis = new FileInputStream(f);
                char[] keystorePassword = password.toCharArray();
                keyStore.load(fis, keystorePassword);
                fis.close();

                // Retrieve the private key
                char[] keyPassword = password.toCharArray();
                String alias = "saml";
                KeyStore.PasswordProtection keyPasswordProtection = new KeyStore.PasswordProtection(keyPassword);
                KeyStore.PrivateKeyEntry privateKeyEntry = (KeyStore.PrivateKeyEntry) keyStore.getEntry(alias, keyPasswordProtection);
                privateKey = privateKeyEntry.getPrivateKey();
                certificate = loadCertificateFromFile(new File(dir,"certificate.cer").getAbsolutePath());
            }

            // Create credentials
            BasicCredential credential = new BasicCredential(certificate.getPublicKey(), privateKey);
            return credential;

        }

    public PrivateKey loadPrivateKeyFromFile(String filePath) throws Exception {
        try (PemReader pemReader = new PemReader(new FileReader(filePath))) {
            PemObject pemObject = pemReader.readPemObject();
            byte[] pemContent = pemObject.getContent();
            PKCS8EncodedKeySpec keySpec = new PKCS8EncodedKeySpec(pemContent);
            KeyFactory keyFactory = KeyFactory.getInstance("RSA");
            return keyFactory.generatePrivate(keySpec);
        }
    }

    public X509Certificate loadCertificateFromFile(String filePath) throws Exception {
        try (FileInputStream fis = new FileInputStream(filePath)) {
            CertificateFactory certFactory = CertificateFactory.getInstance("X.509");
            return (X509Certificate) certFactory.generateCertificate(fis);
        }
    }


    public X509Certificate generateSelfSignedCertificate(KeyPair keyPair,Map<String, String> contextParams) throws Exception {
        // Set the certificate validity period
        Date startDate = new Date();
        long diff = Long.parseLong(getOrDefault(contextParams,"saml-certificateExpirationTime",String.valueOf(2* 365 * 24 * 60 * 60 * 1000L)));
        Date endDate = new Date(startDate.getTime() + diff);

        String certType = getOrDefault(contextParams,"saml-certificateType","SHA256WithRSA");
        // Create a content signer
        ContentSigner contentSigner = new JcaContentSignerBuilder(certType).setProvider("BC").build(keyPair.getPrivate());

        String cnName = getOrDefault(contextParams,"saml-certificateCNName","CN=SAML XPLG SSO");
        // Build the certificate
        X509v3CertificateBuilder certBuilder = new JcaX509v3CertificateBuilder(
                new X500Name(cnName), // issuer
                BigInteger.valueOf(System.currentTimeMillis()), // serial number
                startDate, // start date
                endDate, // end date
                new X500Name(cnName), // subject
                keyPair.getPublic() // public key
        );

        // Convert the certificate builder to a real certificate
        return new JcaX509CertificateConverter().setProvider("BC").getCertificate(certBuilder.build(contentSigner));
    }

    public void saveCertificateToFile(X509Certificate certificate, String filePath) throws Exception {
        try (FileOutputStream fos = new FileOutputStream(filePath)) {
            fos.write(certificate.getEncoded());
        }
    }

    public void savePrivateKeyToFile(PrivateKey privateKey, String filePath) throws IOException {
        PemObject pemObject = new PemObject("PRIVATE KEY", privateKey.getEncoded());
        try (PemWriter pemWriter = new PemWriter(new FileWriter(filePath))) {
            pemWriter.writeObject(pemObject);
        }
    }

    public void savePublicKeyToFile(PublicKey publicKey, String filePath) throws IOException {
        PemObject pemObject = new PemObject("PUBLIC KEY", publicKey.getEncoded());
        try (PemWriter pemWriter = new PemWriter(new FileWriter(filePath))) {
            pemWriter.writeObject(pemObject);
        }
    }





    public String toString(Element xml) throws Exception {
        TransformerFactory tf = TransformerFactory.newInstance();
        Transformer transformer = tf.newTransformer();
        transformer.setOutputProperty(OutputKeys.OMIT_XML_DECLARATION, "yes");
        transformer.setOutputProperty(OutputKeys.INDENT, "yes");
        StringWriter writer = new StringWriter();
        transformer.transform(new DOMSource(xml), new StreamResult(writer));
        return writer.getBuffer().toString();
    }

    public String encodeSAMLRequest(AuthnRequest authnRequest) throws Exception {
        // Marshalling and base64 encoding
        Element authDOM = XMLObjectProviderRegistrySupport.getMarshallerFactory().getMarshaller(authnRequest).marshall(authnRequest);
        String samlRequestXML = toString(authDOM);
        if (ssoCommon.isDebugEnabled()){
            ssoCommon.ssoLogInfo("SAML Request = " + samlRequestXML);
        }
        //System.out.println("samlRequestXML = " + samlRequestXML);
        return Base64.getEncoder().encodeToString(samlRequestXML.getBytes());
    }

    public AuthnRequest buildAuthnRequest(String destination, String acsUrl,String issuerId,String protocolBinding,String format) throws Exception{
        XMLObjectBuilderFactory builderFactory = XMLObjectProviderRegistrySupport.getBuilderFactory();

        AuthnRequestBuilder authnRequestBuilder = (AuthnRequestBuilder) builderFactory.getBuilder(AuthnRequest.DEFAULT_ELEMENT_NAME);
        AuthnRequest authnRequest = authnRequestBuilder.buildObject();

        authnRequest.setID("_" + UUID.randomUUID().toString());
        authnRequest.setIssueInstant(Instant.now());
        authnRequest.setDestination(destination);
        authnRequest.setProtocolBinding(protocolBinding != null && protocolBinding.length() > 0 ? protocolBinding : SAMLConstants.SAML2_POST_BINDING_URI);
        authnRequest.setAssertionConsumerServiceURL(acsUrl);

        if (issuerId != null){
            IssuerBuilder issuerBuilder = (IssuerBuilder) builderFactory.getBuilder(Issuer.DEFAULT_ELEMENT_NAME);
            Issuer issuer = issuerBuilder.buildObject();
            issuer.setValue(issuerId);

            authnRequest.setIssuer(issuer);
        }



        NameIDPolicyBuilder nameIDPolicyBuilder = (NameIDPolicyBuilder) builderFactory.getBuilder(NameIDPolicy.DEFAULT_ELEMENT_NAME);
        NameIDPolicy nameIDPolicy = nameIDPolicyBuilder.buildObject();
        nameIDPolicy.setFormat(format != null && format.length() > 0 ?  format : "urn:oasis:names:tc:SAML:1.1:nameid-format:unspecified");
        nameIDPolicy.setAllowCreate(true);

        authnRequest.setNameIDPolicy(nameIDPolicy);


        return authnRequest;
    }

    public void signRequest(AuthnRequest authnRequest,String keysPath,Map<String, String> contextParams)throws Exception {
        BasicCredential credential = generateBasicCredential(keysPath,contextParams);

        // Build the Signature object
        SignatureImpl signature = new SignatureBuilder().buildObject();
        signature.setSigningCredential(credential);
        signature.setSignatureAlgorithm(SignatureConstants.ALGO_ID_SIGNATURE_RSA_SHA256);
        signature.setCanonicalizationAlgorithm(SignatureConstants.ALGO_ID_C14N_EXCL_OMIT_COMMENTS);

        // Set the Signature on the AuthnRequest
        authnRequest.setSignature(signature);

        SignatureSigningParameters signingParameters = new SignatureSigningParameters();
        signingParameters.setSigningCredential(credential);
        signingParameters.setSignatureAlgorithm(SignatureConstants.ALGO_ID_SIGNATURE_RSA_SHA256);
        signingParameters.setSignatureCanonicalizationAlgorithm(SignatureConstants.ALGO_ID_C14N_EXCL_OMIT_COMMENTS);

        // Sign the AuthnRequest
        try {
            SignatureSupport.signObject(authnRequest, signingParameters);
        } catch (Exception e) {
            throw e;
        }
    }

    public String marshallAndEncode(AuthnRequest authnRequest) throws Exception {
        // Marshall the request to XML
        MarshallerFactory marshallerFactory = XMLObjectProviderRegistrySupport.getMarshallerFactory();
        Marshaller marshaller = marshallerFactory.getMarshaller(authnRequest);
        Element authDOM = marshaller.marshall(authnRequest);

        // Convert the XML to a string and base64 encode it
        StringWriter stringWriter = new StringWriter();
        String xmlString = SerializeSupport.nodeToString(authDOM);

         if (ssoCommon.isDebugEnabled()){
            ssoCommon.ssoLogInfo("SAML Request = " + xmlString);
        }

        //System.out.println("xml string = " + xmlString);

        return Base64.getEncoder().encodeToString(xmlString.getBytes(StandardCharsets.UTF_8));
    }

    public Response decodeAndUnmarshall(String samlResponse) throws Exception{
         byte[] decodedBytes = Base64.getDecoder().decode(samlResponse);

        if (ssoCommon.isInfoEnabled()){
            ssoCommon.ssoLogInfo("SAML Response = " + new String(decodedBytes));
        }
        //System.out.println("reponse = " + new String(decodedBytes));
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        factory.setNamespaceAware(true);
        DocumentBuilder builder = factory.newDocumentBuilder();
        Document document = builder.parse(new ByteArrayInputStream(decodedBytes));

        // Unmarshall the SAML response
        Element rootElement = document.getDocumentElement();
        UnmarshallerFactory unmarshallerFactory = XMLObjectProviderRegistrySupport.getUnmarshallerFactory();
        Unmarshaller unmarshaller = unmarshallerFactory.getUnmarshaller(rootElement);
        Response response = (Response) unmarshaller.unmarshall(rootElement);
        return response;
    }


    public JSONObject parseJsonFile(String path) {
         try (FileReader reader = new FileReader(path)) {
            // Parse the JSON file
            JSONObject jsonObject = (JSONObject) new JSONParser().parse(reader);

            return jsonObject;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    public JSONObject createLoginDetails(JSONObject formObject,Assertion assertion) throws Exception {
        JSONObject attributesMapping = (JSONObject)formObject.get("mapping.attributes");
        if (attributesMapping == null){
            attributesMapping = new JSONObject();
        }

        String configurationDir = (String)formObject.get("plugin.configurationDir");
        if (configurationDir != null && configurationDir.length() > 0){
            File f = new File(new File(configurationDir),"mapping/attributes.json");
            if (f.exists()){
                JSONObject attributesMappingFromFile = parseJsonFile(f.getAbsolutePath());
                if (attributesMappingFromFile != null){
                    attributesMapping.putAll(attributesMappingFromFile);
                }
            }
        }
        configurationDir = (String)formObject.get("plugin.cachePath");
        if (configurationDir != null && configurationDir.length() > 0){
            File f = new File(new File(configurationDir),"attributes.json");
            if (f.exists()){
                JSONObject attributesMappingFromFile = parseJsonFile(f.getAbsolutePath());
                if (attributesMappingFromFile != null){
                    attributesMapping.putAll(attributesMappingFromFile);
                }
            }

        }
        JSONObject realmMapping = (JSONObject)formObject.get("mapping.realm");
        if (realmMapping == null){
            realmMapping = new JSONObject();
        }
        configurationDir = (String)formObject.get("plugin.configurationDir");
        if (configurationDir != null && configurationDir.length() > 0){
            File f = new File(new File(configurationDir),"mapping/realm.json");
            if (f.exists()){
                JSONObject realmMappingFromFile = parseJsonFile(f.getAbsolutePath());
                if (realmMappingFromFile != null){
                    realmMapping.putAll(realmMappingFromFile);
                }
            }
        }
        configurationDir = (String)formObject.get("plugin.cachePath");
        if (configurationDir != null && configurationDir.length() > 0){
            File f = new File(new File(configurationDir),"realm.json");
            if (f.exists()){
                JSONObject realmMappingFromFile = parseJsonFile(f.getAbsolutePath());
                if (realmMappingFromFile != null){
                    realmMapping.putAll(realmMappingFromFile);
                }
            }

        }
        return createLoginDetails(formObject,assertion,attributesMapping,realmMapping);
    }



    public JSONObject createLoginDetails(JSONObject formObject,Assertion assertion,JSONObject attributesMapping,JSONObject realmMapping) throws Exception {
        if (assertion == null) return null;
        JSONObject userInfo = null;

        String username = assertion.getSubject().getNameID().getValue();
        if (username == null){
            return null;
        }


        Map<String,String> attributesMap = new HashMap<>();
        if (attributesMapping != null) {
            for (Object o : attributesMapping.keySet()) {
              Object value = attributesMapping.get(o);
              String key = o.toString();
              if (value instanceof JSONArray){
                  JSONArray arr = (JSONArray)value;
                  for (Object val : arr) {
                      attributesMap.put(val.toString(),key);
                  }
              }else if (value instanceof String){
                  attributesMap.put(value.toString(),key);
              }
            }
        }
        //System.out.println("attributesMap = " + attributesMap);

        JSONObject usersRealm = null;
        JSONObject groupsRealm = null;
        JSONObject policies = null;
        if (realmMapping != null){
            usersRealm = (JSONObject)realmMapping.get("users");
            groupsRealm = (JSONObject)realmMapping.get("groups");
            policies = (JSONObject)realmMapping.get("policies");
        }

        JSONObject loginDetails = new JSONObject();
        userInfo = new JSONObject();
        userInfo.put("name" , username);

        loginDetails.put("user",userInfo);

        JSONArray groupsArr = null;
        JSONObject context = null;

        List<AttributeStatement> attributeStatements = assertion.getAttributeStatements();
        if (attributeStatements != null) {
            for (AttributeStatement attributeStatement : attributeStatements) {
                List<Attribute> attributes = attributeStatement.getAttributes();
                if (attributes != null) {
                    for (Attribute attribute : attributes) {
                        String name = attribute.getName();

                        List<XMLObject> attributeValues = attribute.getAttributeValues();
                        String attType = attributesMap.get(name);

                        //System.out.println("attribute = " + name +  " ," +  attType);
                        if (attType != null && attributeValues != null && !attributeValues.isEmpty()){
                            if (attType.equals("displayName")){
                                String displayName = attributeValues.get(0).getDOM().getTextContent();
                                if (displayName != null && displayName.length() > 0){
                                    userInfo.put("displayName" , displayName);
                                }
                            }else if (attType.equals("groups")){
                                if (context == null){
                                    context = new JSONObject();
                                }
                                if (groupsArr == null){
                                    groupsArr = new JSONArray();
                                    context.put("groups",groupsArr);
                                }
                                for (XMLObject attributeValue : attributeValues) {
                                    String groupId = attributeValue.getDOM().getTextContent();
                                    JSONObject groupObj = new JSONObject();
                                    groupObj.put("name",groupId);
                                    if (groupsRealm != null){
                                        JSONObject obj = (JSONObject)groupsRealm.get(groupId);
                                        if (obj != null){
                                            groupObj = obj;
                                        }
                                    }
                                    groupsArr.add(groupObj);
                                }
                            }
                        }



                        if (attributes != null) {
                            for (XMLObject attributeValue : attributeValues) {
                                //System.out.println("\t\t" + attributeValue.getDOM().getTextContent());
                            }
                        }
                    }
                }
            }
        }
        if (context != null){
            loginDetails.put("context",context);
        }

        if (usersRealm != null){
            JSONObject userRealm = (JSONObject)usersRealm.get(username);
            if (userRealm != null){
                loginDetails.putAll(userRealm);
            }
        }
        return loginDetails;
    }
}
%>
