<%!

void fetchPushCommitEvents(
	String organization,
	HttpClient client,
	String clientAuth,
	JSONObject pluginRefObj,
	JSONArray array,
	Instant sinceTimeDefault,
	int numOfEvents,
	boolean toFetchCommitDetail,
	boolean toRetrieveCommitFiles
) {
	Map<String, Instant> repoTimeMap = new HashMap<>();
	JSONObject repoTimesJSONObject = (JSONObject)pluginRefObj.get("repoTimes");
	if (repoTimesJSONObject != null) {
		for (Map.Entry<String, String> entry : (Iterable<Map.Entry<String, String>>)repoTimesJSONObject.entrySet()) {
			Instant instant = null;
			String instantS = entry.getValue();
			if (instantS != null) {
				instant = Instant.parse(instantS);
			}
			repoTimeMap.put(entry.getKey(), instant);
		}
	}

	Set<String> reposToDelete = new HashSet<>(repoTimeMap.keySet());
	for (JSONObject repoObj :
		new GitHubJSONObjectIterable(client, clientAuth, "/orgs/" + organization + "/repos?sort=created&direction=asc"))
	{
		String repoName = (String)repoObj.get("name");
		repoTimeMap.putIfAbsent(repoName, null);
		reposToDelete.remove(repoName);
	}
	repoTimeMap.keySet().removeAll(reposToDelete);

	for (Map.Entry<String, Instant> entry : repoTimeMap.entrySet()) {
		String repo = entry.getKey();
		Instant sinceTime = entry.getValue();
		if (sinceTime == null) {
			sinceTime = sinceTimeDefault;
		}

		// NOTE a big assumption here: GH lists pushes sorted by their 'created_at', descending;
		//		the idea is to go page by page until the end -- or until "since" time is reached
		//		while collecting all the pushes' commits (having push-commit relationship inverted,
		//		so .payload.commits in a push will become .push in those commits) in a list
		//		in reverse (trim that list by whole pushes if it grows too large -- so the operation
		//		will have some memory limits -- another assumption here is a single push will
		//		not have more commits than that list's size maximum); and then, base the result
		//		on this according to log type

		// NOTE for short(ish, up to 1000 elements), assuming ArrayList is efficient enough
		List<JSONObject> pushObjects = new ArrayList<JSONObject>();
		for (JSONObject pushObject :
			new GitHubJSONObjectIterable(client, clientAuth, "/repos/" + organization + "/" + repo + "/events"))
		{
			if (!"PushEvent".equals((String)pushObject.get("type"))) {
				continue;
			}
			if (sinceTime != null && !pushTime(pushObject).isAfter(sinceTime)) {
				break;
			}
			// NOTE an approximation, will actually yield "slightly" more than max
			if (array.size() + pushObjects.size() >= numOfEvents) {
				pushObjects.remove(pushObjects.size() - 1);
			}
			pushObjects.add(0, pushObject);
		}
		for (JSONObject pushObject : pushObjects) {
			JSONObject payload = (JSONObject)pushObject.get("payload");
			for (Object cObj : (JSONArray)payload.remove("commits")) {
				JSONObject commitShortObject = (JSONObject)cObj;

				if (toRetrieveCommitFiles) {
					for (JSONObject commitObject :
						new GitHubJSONObjectIterable(client, clientAuth, (String)commitShortObject.get("url")))
					{
						commitObject.put("push", pushObject);
						for (Object fObj : (JSONArray)commitObject.remove("files")) {
							JSONObject fileObject = (JSONObject)fObj;
							fileObject.remove("patch");
							fileObject.put("commit", commitObject);
							array.add(fileObject.toString());
						}
					}
				} else if (toFetchCommitDetail) {
					String sha = (String)commitShortObject.get("sha");
					for (JSONObject commitObject :
						new GitHubJSONObjectIterable(client, clientAuth,
							"/repos/" + organization + "/" + repo + "/commits?sha=" + sha
							+ "&until=" + pushTime(pushObject).toString(), 10))
					{
						if (sha.equals(commitObject.get("sha"))) {
							commitObject.put("push", pushObject);
							array.add(commitObject.toString());
							break;
						}  // NOTE in case of a removed commit, we'll be walking a very long way...
					}
				} else {
					commitShortObject.put("push", pushObject);
					array.add(commitShortObject.toString());
				}
			}
		}

		if (pushObjects.size() != 0) {
			// NOTE the assumption here is no two pushes happen at the same time
			repoTimeMap.put(repo, pushTime(pushObjects.get(pushObjects.size() - 1)));
		}
		if (array.size() >= numOfEvents) {
			break;
		}
	}

	Map<String, String> repoTimeS = new HashMap<>();
	for (Map.Entry<String, Instant> entry : repoTimeMap.entrySet()) {
		Instant instant = entry.getValue();
		String instantS = null;
		if (instant != null) {
			instantS = instant.toString();
		}
		repoTimeS.put(entry.getKey(), instantS);
	}
	pluginRefObj.put("repoTimes", repoTimeS);
}

%>
