<%@ page language="java" contentType="application/json; charset=UTF-8" pageEncoding="UTF-8"%>
<%@ page import="java.time.Instant" %>
<%@ page import="java.time.temporal.ChronoUnit" %>
<%@ page import="org.json.simple.JSONObject" %>
<%@ page import="org.json.simple.JSONArray" %>
<%@ page import="org.json.simple.parser.JSONParser" %>

<%@ include file="../../../../common/rest/restCommonServices.jsp" %>
<%@ include file="GitHubUtil.jsp" %>
<%@ include file="fetchPushCommitEvents.jsp" %>
<%@ include file="fetchAuditEvents.jsp" %>
<%

if (!checkSecurity(request)) {
	out.println(buildErrorResult("Rest is not authorized", null, 401));
	response.setStatus(401);
	return;
}

String pluginToken = request.getParameter("plugin-token");
if (pluginToken == null || pluginToken.length() == 0) {
	pluginToken = request.getHeader("plugin-token");
}
if (pluginToken == null) {
	pluginToken = "";
}

StringBuilder requestBody = new StringBuilder();
String line;
while ((line = request.getReader().readLine()) != null) {
	requestBody.append(line);
}
if (requestBody.length() == 0) {
	requestBody.append("{}");
}
JSONParser jsonParser = new JSONParser();
JSONObject jsonObject = (JSONObject)jsonParser.parse(requestBody.toString());

boolean scripttest = Boolean.TRUE.equals(jsonObject.get("scripttest"));

JSONObject metaObj = (JSONObject)jsonObject.get("metaData");
String pluginDataRef = request.getParameter("pluginDataRef");
if (metaObj != null && pluginDataRef == null) {
	pluginDataRef = (String)metaObj.get("pluginDataRef");
}

JSONObject initObject = (JSONObject)session.getAttribute(pluginToken);
if (initObject == null) {
	out.println(buildErrorResult("Init bad", null, 417));
	response.setStatus(417);
	return;
}

Instant sinceTimeDefault = null;
Number sinceTimeDefaultNum = (Number)initObject.get("startTS");
if (sinceTimeDefaultNum != null) {	// NOTE not supposed to be null anyway though
	sinceTimeDefault = Instant.ofEpochSecond(sinceTimeDefaultNum.longValue(), 0);
}

// NOTE will return numOfEvents, more or less;
//		assuming no repo contains pushes with more commits than this (easy assumption);
//		this default is also a reasonable number to keep the memory consumption low;
//		may return "a bit" more, since it will chunk whole pushes of commits, and,
//		in case of file-change log entries, full commits of file changes
int numOfEvents = 1000;
JSONObject queryObject = (JSONObject)initObject.get("queryInfo");
if (queryObject != null) {	// NOTE not exactly the num of log entries, but it will do
	Number numOfEventsNum = (Number)queryObject.get("numOfEvents");
	if (numOfEventsNum != null) {
		numOfEvents = numOfEventsNum.intValue();
	}
}
String numOfEventsS = request.getParameter("numOfEvents");
if (numOfEventsS != null && numOfEventsS.length() != 0) {
	numOfEvents = Integer.parseInt(numOfEventsS);
}

String clientAuth = (String)initObject.get("clientAuth");
if (clientAuth == null){
	out.println(buildErrorResult("Auth bad", null, 417));
	response.setStatus(417);
	return;
}

String organization = (String)initObject.get("organization");

String logType = request.getParameter("logType");
if (logType == null || logType.length() == 0) {
	logType = (String)initObject.get("logType");
}
if (logType == null) {
	logType = "github_org_audit";
}
boolean toReturnPushCommitNotAudit = true;
// NOTE if this is false, the result may contain removed commits
boolean toFetchCommitDetail = false;
boolean toRetrieveCommitFiles = false;	// NOTE if true, implies toFetchCommitDetail
if (logType.equals("github_org_audit")) {
	toReturnPushCommitNotAudit = false;
} else if (logType.equals("github_org_push_commit")) {
	toFetchCommitDetail = true;
} else if (logType.equals("github_org_push_commit_file")) {
	toRetrieveCommitFiles = true;
} else if (!logType.equals("github_org_push_commit_brief")) {
	out.println(buildErrorResult("Unsupported log type", null, 400));
	response.setStatus(400);
	return;
}


JSONObject pluginRefObj = null;
if (pluginDataRef != null) {
	pluginRefObj = (JSONObject)jsonParser.parse(pluginDataRef);
}
if (pluginRefObj == null) {
	pluginRefObj = new JSONObject();
}

JSONArray array = new JSONArray();

try {
	HttpClient client = createHttpsClient();

	if (toReturnPushCommitNotAudit) {
		fetchPushCommitEvents(
			organization,
			client,
			clientAuth,
			pluginRefObj,
			array,
			sinceTimeDefault,
			numOfEvents,
			toFetchCommitDetail,
			toRetrieveCommitFiles
		);
	} else {
		fetchAuditEvents(organization,
			client,
			clientAuth,
			pluginRefObj,
			array,
			sinceTimeDefault,
			numOfEvents
		);
	}
} catch (Exception e) {
	// TODO look for the better method maybe
	xpolog.eye.error.ErrorModel.log.error("Failed to get next events for " + initObject.toString(), e);

	if (scripttest){
		response.setStatus(200);
	} else {
		response.setStatus(500);
	}

	out.println(buildErrorResult(null, e));
	return;
}

%>
{
	"metaData" : {
		"pluginDataRef" : "<%=JSONObject.escape(pluginRefObj.toString())%>"
	},
	"dataEvents" : <%=array.toString()%>
}
